<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * XtdirControllerPromoentries
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
*/
class XtdirControllerPromotions extends FOFController
{
	/**
	 * getCategories
	 *
	 * @return	void
	 */
	public function getCategories()
	{
		$truncate_cats = EParameter::getComponentParam(CXTDIR, 'truncate_cats', UtilHelper::CATS_TEXT_LENGHT);

		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$section_id = $this->input->get('section_id');

		$categoriesModel = FOFModel::getTmpInstance('Objects', 'XtdirModel');
		$categoriesModel->setState('state', 1);
		$categoriesModel->setState('oType', 'category');

		if ($section_id)
		{
			$categoriesModel->setState('section_id', $section_id);
		}

		$categoriesModel->setState('filter_order', 'name');
		$items = $categoriesModel->getItemList(true);
		$categories = array();

		if (!empty($items))
		{
			foreach ($items as $item)
			{
				$name = $item->name;

				if ($truncate_cats)
				{
					$name = UtilHelper::truncString($name, $truncate_cats, true);
				}

				$c = array(
								'id' => $item->id,
								'name' => UtilHelper::truncString($name, 20, true)
				);
				$categories[] = $c;
			}
		}

		$message = json_encode(
				array(
						'status' => true,
						'error_message' => 'Ok',
						'items' => $categories
			)
		);

		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * getFields
	 *
	 * @return	void
	 */
	public function getFields()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$section_id = $this->input->get('section_id');

		$fieldsModel = FOFModel::getTmpInstance('Fields', 'XtdirModel');
		$fieldsModel->setState('enabled', 1);
		$fieldsModel->setState('filter_order', 'section,position');

		if ($section_id)
		{
			$fieldsModel->setState('section', $section_id);
		}

		$items = $fieldsModel->getItemList(true);
		$fields = array();

		if (!empty($items))
		{
			foreach ($items as $item)
			{
				$f = array(
								'id' => $item->fid,
								'name' => $item->nid
				);
				$fields[] = $f;
			}
		}

		$message = json_encode(
				array(
						'status' => true,
						'error_message' => 'Ok',
						'items' => $fields
			)
		);

		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * Saves the order of the items
	 *
	 * @return  void
	 */
	public function saveorder()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$model = $this->getThisModel();

		if (!$model->getId())
		{
			$model->setIDsFromRequest();
		}

		$ids = $model->getIds();
		$orders = $this->input->get('order', array(), 'array');

		if ($n = count($ids))
		{
			for ($i = 0; $i < $n; $i++)
			{
				$model->setId($ids[$i]);
				$neworder = (int) $orders[$i];

				$item = $model->getItem();
				$key = $item->getKeyName();

				if ($item->$key == $ids[$i])
				{
					$item->ordering = $neworder;
					$model->save($item);
				}
			}
		}

		// $status = $model->reorder();

		// Redirect

		if ($customURL = $this->input->get('returnurl', '', 'string'))
		{
			$customURL = base64_decode($customURL);
		}

		$url = !empty($customURL) ? $customURL : 'index.php?option=' . $this->component . '&view=' . FOFInflector::pluralize($this->view);
		$this->setRedirect($url);

		return $status;
	}

	/**
	 * index.
	 *
	 * @return	void
	 */
	public function index()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		TreeIndexer::index(true);

		// Redirect
		if ($customURL = $this->input->get('returnurl', '', 'string'))
		{
			$customURL = base64_decode($customURL);
		}

		// ' . FOFInflector::pluralize($this->view)
		$url = !empty($customURL) ? $customURL : 'index.php?option=' . $this->component . '&view=objects';
		$this->setRedirect($url);
	}
}
