<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

include_once JPATH_ADMINISTRATOR . '/components/com_xtdir/api/sobipro_api.php';

/**
 * XtdirControllerSystems
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
*/
class XtdirControllerSystems extends FOFController
{
	/**
	 * Creates a new model object
	 *
	 * @param   string  $name    The name of the model class, e.g. Items
	 * @param   string  $prefix  The prefix of the model class, e.g. FoobarModel
	 * @param   array   $config  The configuration parameters for the model class
	 *
	 * @return  FOFModel  The model object
	 */
	protected function createModel($name, $prefix = '', $config = array())
	{
		$result = null;

		$modelName = 'Extensions';
		$classPrefix = 'ExtlyModel';

		$result = FOFModel::getAnInstance($modelName, $classPrefix, $config);

		return $result;
	}

	/**
	 * index.
	 *
	 * @return	void
	 */
	public function index()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		TreeIndexer::index(true);

		// Redirect
		if ($customURL = $this->input->get('returnurl', '', 'string'))
		{
			$customURL = base64_decode($customURL);
		}

		$url = !empty($customURL) ? $customURL : 'index.php?option=' . $this->component . '&view=' . FOFInflector::pluralize($this->view);
		$this->setRedirect($url);

		JFactory::getApplication()->enqueueMessage(JText::_('COM_XTDIR_VIEW_SYSTEM_INDEX_SUCCESS'), 'message');
	}

	/**
	 * Execute something after applySave has run.
	 *
	 * @return  boolean  True to allow normal return, false to cause a 403 error
	 */
	protected function onAfterApplySave()
	{
		EExtensionHelper::cleanCache();

		return parent::onAfterApplySave();
	}

	/**
	 * Save the incoming data and then return to the Edit task
	 *
	 * @return  void
	 */
	public function apply()
	{
		$this->input->set('returnurl', '');
		parent::apply();
	}

	/**
	 * getIndexBegin
	 *
	 * @return	void
	 */
	public function getIndexBegin()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		include_once JPATH_ADMINISTRATOR . '/components/com_xtdir/api/sobipro_api.php';

		$message = array(
						'status' => false,
						'error_message' => JText::_('COM_XTDIR_VIEW_INDEX_FAILED_ERR')
		);

		try
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::INFO, "getIndexBegin");
			$treeIndexer = new TreeIndexer;
			$treeIndexer->rebuildInitialization();

			$catalogs = $treeIndexer->getCatalogs();
			$message = $this->_getIndexBeginMessage($catalogs);
		}
		catch (Exception $e)
		{
			$message['status'] = false;
			$message['error_message'] .= ' Start ' . $e->getMessage();
		}

		$message = json_encode($message);
		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * getIndexStatus
	 *
	 * @return	void
	 */
	public function getIndexStatus()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		include_once JPATH_ADMINISTRATOR . '/components/com_xtdir/api/sobipro_api.php';

		$message = array(
						'status' => false,
						'error_message' => JText::_('COM_XTDIR_VIEW_INDEX_FAILED_ERR')
		);

		try
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::INFO, "getIndexStatus");

			$section_id = $this->input->get('sectionId', null, 'int');
			$continue = $this->input->get('isContinue', 0, 'int');

			$treeIndexer = new TreeIndexer;
			$completed = $treeIndexer->rebuildCatalog($section_id, $continue);
			$message = array(
							'status' => true,
							'completed' => $completed
			);

			$message['total'] = number_format($treeIndexer->getTotal($section_id));
		}
		catch (Exception $e)
		{
			$message['status'] = false;
			$message['error_message'] .= ' Status ' . $e->getMessage();
		}

		$message = json_encode($message);
		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * getIndexBegin
	 *
	 * @return	void
	 */
	public function getIndexEnd()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$message = array(
						'status' => false,
						'error_message' => JText::_('COM_XTDIR_VIEW_INDEX_FAILED_ERR')
		);

		try
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::INFO, "getIndexEnd");
			$treeIndexer = new TreeIndexer;
			$treeIndexer->rebuildFinalization();

			$message = array(
							'status' => true,
							'error_message' => 'ok'
			);
		}
		catch (Exception $e)
		{
			$message['status'] = false;
			$message['error_message'] .= ' End ' . $e->getMessage();
		}

		$message = json_encode($message);
		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * _getIndexBeginMessage
	 *
	 * @param   array  $catalogs  Params
	 *
	 * @return	object
	 */
	private function _getIndexBeginMessage($catalogs)
	{
		$message = array(
						'status' => true,
						'error_message' => 'Ok'
		);

		$sections = array();

		foreach ($catalogs as $catalog_id)
		{
			$object = SobiproHelper::loadAndExtend($catalog_id);

			$section = new StdClass;
			$section->id = $catalog_id;
			$section->name = $object->get('name');

			$sections[] = $section;
		}

		$message['catalogs'] = $sections;

		return $message;
	}

	/**
	 * installRouterApp
	 *
	 * @return	void
	 */
	public function installRouterApp()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		SobiproHelper::installApp('/media/com_xtdir/apps/app_xtd_router.zip');
	}

	/**
	 * installCategoriesFilterApp
	 *
	 * @return	void
	 */
	public function installCategoriesFilterApp()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		SobiproHelper::installApp('/media/com_xtdir/apps/app_xtd_categoriesfilter.zip');
	}
}
