<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');
jimport('joomla.error.error');

/**
 * PromotedHelper Helper class.
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
 */
class PromotedHelper
{
	const ENTRY_PROMOTED = 1;
	const ENTRY_NOT_PROMOTED = 999999;
	const SORTTYPES_DEFAULT = 'D';
	const SORTTYPES_RANDOM = 'R';
	const SORTTYPES_HITS = 'H';
	const SORTTYPES_MULTI_ARMED_BANDIT = 'M';

	/**
	 * index
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return  void
	 *
	 * @since   1.0
	 */
	public static function index($entry_id = null)
	{
		$logger = XtdirLogger::getInstance();

		$time = microtime(true);
		$logger->log(JLog::INFO, 'Promoted Index start');

		if (defined('XTDIR_CRONJOB_RUNNING'))
		{
			echo JProfiler::getInstance()->mark('Promoted Index start') . "\n";
		}

		self::indexTruncate($entry_id);
		self::updateLastrun($entry_id);
		self::updateCategoryPromoted($entry_id);
		self::updateSectionPromoted($entry_id);

		self::updateSectionRelations($entry_id);
		self::updateCategoryRelations($entry_id);

		$suborder_field = EParameter::getComponentParam('com_xtdir', 'suborder_field');

		if ($suborder_field)
		{
			self::updateSubOrder($entry_id);
		}

		self::updatePromoInformationField($entry_id);

		$logger->log(JLog::INFO, 'Promoted Index End - ' . round(microtime(true) - $time, 3) . ' seconds');

		if (defined('XTDIR_CRONJOB_RUNNING'))
		{
			echo JProfiler::getInstance()->mark('Promoted Index End') . "\n";
		}
	}

	/**
	 * indexTruncate
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return  void
	 */
	private static function indexTruncate($entry_id = null)
	{
		$_db = JFactory::getDbo();

		if ($entry_id)
		{
			$entry_idq = $_db->q($entry_id);
			$query = "DELETE FROM `#__xtdir_final_categorypromoted` WHERE entry_id = " . $entry_idq;
			$_db->setQuery($query);
			$_db->execute();

			$query = "DELETE FROM `#__xtdir_final_sectionpromoted` WHERE entry_id = " . $entry_idq;
			$_db->setQuery($query);
			$_db->execute();
		}
		else
		{
			$query = "TRUNCATE `#__xtdir_final_categorypromoted`;";
			$_db->setQuery($query);
			$_db->execute();

			$query = "TRUNCATE `#__xtdir_final_sectionpromoted`;";
			$_db->setQuery($query);
			$_db->execute();
		}
	}

	/**
	 * updateLastrun
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return  void
	 */
	private static function updateLastrun($entry_id = null)
	{
		$_db = JFactory::getDbo();

		$nowSql = $_db->q(JFactory::getDate()->toSql());
		$query = 'UPDATE `#__xtdir_lastrun` SET `lastrun` = ' . $nowSql;
		$_db->setQuery($query);

		try
		{
			$_db->execute();

			if ($error = $_db->getErrorMsg())
			{
				$logger->log(JLog::ERROR, $error);
			}
		}
		catch (Exception $e)
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());
		}
	}

	/**
	 * updateCategoryPromoted
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return  void
	 */
	private static function updateCategoryPromoted($entry_id = null)
	{
		$_db = JFactory::getDbo();

		$query = "INSERT INTO `#__xtdir_final_categorypromoted` (`section_id`, `category_id`, `entry_id`, `ordering`, `promotype_id`,`tiertype_id`,`counter`,`promoentry_id`,`promoorder_id`)
				SELECT `section_id`,`category_id`,`entry_id`,`ordering`,`promotype_id`,`tiertype_id`,`counter`,`promoentry_id`,`promoorder_id` FROM `#__xtdir_final_categorypromoted_view`";

		if ($entry_id)
		{
			$entry_idq = $_db->q($entry_id);
			$query = $query . ' WHERE entry_id = ' . $entry_idq;
		}

		$_db->setQuery($query);

		try
		{
			$_db->execute();

			if ($error = $_db->getErrorMsg())
			{
				$logger = XtdirLogger::getInstance();
				$logger->log(JLog::ERROR, $error);
			}
		}
		catch (Exception $e)
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());
		}
	}

	/**
	 * updateSectionPromoted
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return  void
	 */
	private static function updateSectionPromoted($entry_id = null)
	{
		$_db = JFactory::getDbo();
		$query = "INSERT INTO `#__xtdir_final_sectionpromoted` (`entry_id`,`promotype_id`,`tiertype_id`,`counter`,`ordering`,`promoentry_id`,`promoorder_id`)
				SELECT DISTINCT `entry_id`,`promotype_id`,`tiertype_id`,`counter`,`ordering`,`promoentry_id`,`promoorder_id` FROM `#__xtdir_final_sectionpromoted_view` WHERE promotype_id in ('S','E')";

		if ($entry_id)
		{
			$entry_idq = $_db->q($entry_id);
			$query = $query . ' AND entry_id = ' . $entry_idq;
		}

		$_db->setQuery($query);

		try
		{
			$_db->execute();

			if ($error = $_db->getErrorMsg())
			{
				$logger = XtdirLogger::getInstance();
				$logger->log(JLog::ERROR, $error);
			}
		}
		catch (Exception $e)
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());
		}
	}

	/**
	 * updateSectionRelations
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return  void
	 */
	private static function updateSectionRelations($entry_id = null)
	{
		$_db = JFactory::getDbo();

		// Section relations
		$query = "UPDATE `#__sobipro_relations` r, `#__xtdir_final_sectionpromoted` pe SET r.position = pe.ordering
				WHERE r.id = pe.entry_id";

		if ($entry_id)
		{
			$entry_idq = $_db->q($entry_id);
			$query = $query . ' AND pe.entry_id = ' . $entry_idq;
		}

		$_db->setQuery($query);

		try
		{
			$_db->execute();

			if ($error = $_db->getErrorMsg())
			{
				$logger = XtdirLogger::getInstance();
				$logger->log(JLog::ERROR, $error);
			}
		}
		catch (Exception $e)
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());
		}
	}

	/**
	 * updateCategoryRelations
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return  void
	 */
	private static function updateCategoryRelations($entry_id = null)
	{
		$_db = JFactory::getDbo();

		// Category relations
		$query = "UPDATE `#__sobipro_relations` r, `#__xtdir_final_categorypromoted` pe SET r.position = pe.ordering
				WHERE r.id = pe.entry_id AND r.pid = pe.category_id";

		if ($entry_id)
		{
			$entry_idq = $_db->q($entry_id);
			$query = $query . ' AND pe.entry_id = ' . $entry_idq;
		}

		$_db->setQuery($query);

		try
		{
			$_db->execute();

			if ($error = $_db->getErrorMsg())
			{
				$logger = XtdirLogger::getInstance();
				$logger->log(JLog::ERROR, $error);
			}
		}
		catch (Exception $e)
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());
		}
	}

	/**
	 * updateSubOrder
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return  void
	 */
	private static function updateSubOrder($entry_id = null)
	{
		$_db = JFactory::getDbo();
		$dbproxy = EDbProxyHelper::getInstance();

		$entryModel = FOFModel::getTmpInstance('Objects', 'XtdirModel');
		$entryModel->setState('state', 1);
		$entryModel->setState('oType', 'entry');
		$total = $entryModel->getTotal();

		$power = round(log10($total), 0, PHP_ROUND_HALF_DOWN) + 1;
		$base = pow(10, $power);

		if ($entry_id)
		{
			$entry_idq = $_db->q($entry_id);

			self::updateSubOrderBase($_db, $dbproxy, $entry_id, $base);
			$dbproxy->flushUpdates();
		}
		else
		{
			$suborder_field = EParameter::getComponentParam('com_xtdir', 'suborder_field');

			$query = $_db->getQuery(true);
			$query->select('distinct r.id, r.position');
			$query->from('#__sobipro_object as o');
			$query->from('#__sobipro_relations as r');
			$query->from('#__sobipro_field_data as d');

			$query->where('o.oType = ' . $_db->q('entry'));
			$query->where('o.id = r.id');
			$query->where('o.id = d.sid');
			$query->where('d.fid = ' . $_db->q($suborder_field));

			$query->order('d.baseData');

			$_db->setQuery($query);
			$results = $_db->loadAssocList();

			if (empty($results))
			{
				return;
			}

			$c = count($results);

			for ($i = 0; $i < $c; $i++)
			{
				$row = $results[$i];
				$id = $row['id'];
				self::updateSubOrderBase($_db, $dbproxy, $id, $base, $i);
			}

			$dbproxy->flushUpdates();
		}
	}

	/**
	 * updateSubOrderBase
	 *
	 * @param   object  &$_db      Params
	 * @param   object  &$dbproxy  Params
	 * @param   int     $entry_id  Params
	 * @param   int     $base      Params
	 * @param   int     $seq       Params
	 *
	 * @return  void
	 */
	private static function updateSubOrderBase(&$_db, &$dbproxy, $entry_id, $base, $seq = 0)
	{
		$pq = $_db->qn('position');
		$values = array(
			'position' => '(' . $pq . ' * ' . $base . ' + ' . $seq . ')'
		);

		$dbproxy->addUpdate('#__sobipro_relations', 'id', $entry_id, $values);
	}

	/**
	 * updatePromoInformationField
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return  void
	 */
	private static function updatePromoInformationField($entry_id = null)
	{
		$_db = JFactory::getDbo();

		// Generate Promo Information

		try
		{
			$query = "DELETE FROM `#__sobipro_field_data` WHERE `fid` IN (SELECT `fid` FROM `#__sobipro_field` WHERE `fieldType` = 'promoinformation');";
			$_db->setQuery($query);
			$_db->execute();
		}
		catch (Exception $e)
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());
		}

		$query = <<<SQL
REPLACE INTO `#__sobipro_field_data` (
	`publishUp`,`publishDown`,
	`fid`,`sid`,`section`,`lang`,`enabled`,`baseData`,`approved`,`confirmed`,
	`createdTime`,`createdBy`,`createdIP`,
	`updatedTime`,`updatedBy`,`updatedIP`,
	`copy`)
SELECT DISTINCT
	'0000-00-00 00:00:00', '0000-00-00 00:00:00',
	`fid`, `entry_id`, `section_id`,
	'*', 1,
	`r`.`position`,
	1,0,
	NOW(),0,0,
	NOW(),0,0,
	0
FROM
	`#__xtdir_final_sectionpromoted_view` as `v`,
	`#__sobipro_object` as `o`,
	`#__sobipro_field` as `f`,
	`#__sobipro_relations` as `r`
WHERE
	`v`.`promotype_id` in ('S','E') AND
	`v`.`entry_id` = `o`.`id` AND
	`v`.`section_id` = `f`.`section` AND
	`f`.`fieldType` = 'promoinformation' AND
	`v`.`entry_id` = `r`.`id` AND
	`r`.`oType` = 'entry';
SQL;

		try
		{
			$_db->setQuery($query);
			$_db->execute();
		}
		catch (Exception $e)
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());
		}
	}

	/**
	 * entryTogglePromoted
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return	void
	 */
	public static function entryTogglePromoted($entry_id)
	{
		$now = JFactory::getDate();
		$now_sql = $now->toSql();

		FOFModel::getTmpInstance('TierTypes', 'XtdirModel');

		$promoEntries = FOFModel::getTmpInstance('PromoEntries', 'XtdirModel');
		$promoEntries->setState('entry_id', $entry_id);
		$promoEntry = $promoEntries->getFirstItem(true);

		if ($promoEntry->id)
		{
			$promoEntry->published = !$promoEntry->published;

			if ($promoEntry->published)
			{
				$promoEntry->promoted_since = $now_sql;
				$promoEntry->promoted_until = '0000-00-00 00:00:00';

				// Self::getExpiration($entry_id, XtdirModelTierTypes::TIERTYPES_MANUAL);
			}
		}
		else
		{
			// Never promoted, new promoted entry
			$promoEntry = array();

			// $promoEntry['category_id'] = $category_id;
			$promoEntry['entry_id'] = $entry_id;
			$promoEntry['promoted_since'] = $now_sql;
			$promoEntry['promoted_until'] = '0000-00-00 00:00:00';
			$promoEntry['created'] = $now_sql;
			$promoEntry['created_by'] = JFactory::getUser()->id;
			$promoEntry['ordering'] = self::ENTRY_PROMOTED;
			$promoEntry['published'] = 1;

			// Self::getExpiration($entry_id, XtdirModelTierTypes::TIERTYPES_MANUAL);
		}

		$promoEntries->save($promoEntry);
	}

	/**
	 * isPromoted
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return	void
	 */
	public static function isPromoted($entry_id)
	{
		$finalCategoryPromoted = FOFModel::getTmpInstance('FinalCategoryPromoted', 'XtdirModel');
		$finalCategoryPromoted->setState('entry_id', $entry_id);
		$finalPromotedEntry = $finalCategoryPromoted->getFirstItem(true);

		if (!isset($finalPromotedEntry->id))
		{
			$finalSectionPromoted = FOFModel::getTmpInstance('FinalSectionPromoted', 'XtdirModel');
			$finalSectionPromoted->setState('entry_id', $entry_id);
			$finalPromotedEntry = $finalSectionPromoted->getFirstItem(true);
		}

		return $finalPromotedEntry->id;
	}

	/**
	 * getFinalPromoted
	 *
	 * @param   array  &$entries     Params
	 * @param   int    $section_id   Params
	 * @param   int    $category_id  Params
	 *
	 * @return	void
	 */
	public static function getFinalPromoted(&$entries, $section_id, $category_id = null)
	{
		if (!empty($entries))
		{
			$entrieslist = implode(',', $entries);
		}

		$entries = array();

		if ($category_id)
		{
			$finalCategoryPromoted = FOFModel::getTmpInstance('FinalCategoryPromoted', 'XtdirModel');
			$finalCategoryPromoted->setState('entries', $entrieslist);
			$finalCategoryPromoted->setState('section_id', $section_id);
			$finalCategoryPromoted->setState('category_id', $category_id);
			$finalCategoryPromoted->setState('filter_order', 'ordering, entry_id');
			$finalCategoryPromoted->setState('filter_order_Dir', '');
			$entries = $finalCategoryPromoted->getItemList(true);
		}

		if ((!$category_id) || (empty($entries)))
		{
			$finalSectionPromoted = FOFModel::getTmpInstance('FinalSectionPromoted', 'XtdirModel');
			$finalSectionPromoted->setState('entries', $entrieslist);
			$finalSectionPromoted->setState('section_id', $section_id);
			$finalSectionPromoted->setState('filter_order', 'ordering, entry_id');
			$finalSectionPromoted->setState('filter_order_Dir', '');
			$entries = $finalSectionPromoted->getItemList(true);
		}

		return $entries;
	}

	/**
	 * execute
	 *
	 * @param   mixed  $results      Params
	 * @param   int    $section_id   Params
	 * @param   int    $category_id  Params
	 *
	 * @return	mixed
	 */
	public static function execute($results, $section_id, $category_id)
	{
		// Param normalization
		if (is_array($results))
		{
			$return_array = true;
			$ids = implode(',', $results);
		}
		else
		{
			$results = preg_replace('/[^,0-9]/', '', $results);
			$return_array = false;
			$ids = $results;
			$results = explode(',', $results);
		}
		// $results array()
		// $ids 1,2,3,4,5

		$ordering_entries = self::_getPromotedEntries($results, $ids, $section_id, $category_id);

		$logger = XtdirLogger::getInstance();
		$logger->log(JLog::INFO, 'Promoted entries:', $ordering_entries);

		$clustered_entries = self::_getClusteredEntries($results, $ordering_entries);

		$logger->log(JLog::INFO, 'Clustered entries:', $clustered_entries);

		$not_indexed_entries = array_diff($results, $clustered_entries);
		$final_entries = array_merge($clustered_entries, $not_indexed_entries);

		$logger->log(JLog::INFO, 'Final entries:', $final_entries);

		// Return normalization
		if ($return_array)
		{
			return $final_entries;
		}
		else
		{
			$final = implode(',', $final_entries);

			return $final;
		}
	}

	/**
	 * _getPromotedEntries
	 *
	 * @param   array   $results      Params
	 * @param   string  $ids          Params
	 * @param   int     $section_id   Params
	 * @param   int     $category_id  Params
	 *
	 * @return	array
	 */
	private static function _getPromotedEntries($results, $ids, $section_id, $category_id)
	{
		$_db = JFactory::getDbo();
		$_db->setQuery("SET OPTION SQL_BIG_SELECTS=1");
		$_db->execute();

		$result_category = array();

		if ($category_id)
		{
			$queryc = $_db->getQuery(true);
			$queryc->select('cp.entry_id,cp.ordering,cp.counter');

			$queryc->from('#__xtdir_final_categorypromoted as cp');
			$queryc->from('#__sobipro_object as o');
			$queryc->from('#__sobipro_relations as r');

			$queryc->where('cp.entry_id in (' . $ids . ')');
			$queryc->where('cp.category_id = ' . $_db->q($category_id));
			$queryc->where('cp.entry_id = o.id');

			$queryc->where('r.id = cp.entry_id');
			$queryc->where('r.pid = cp.category_id');
			$queryc->where('r.oType = \'entry\'');

			$queryc->order('r.position');

			// $logger = XtdirLogger::getInstance();
			// $logger->log(JLog::INFO, '_getPromotedEntries queryc: ' . $queryc->__toString());

			$_db->setQuery($queryc, 0, 0);
			$result_category = $_db->loadObjectList('entry_id');
		}

		$result_category_keys = array_keys($result_category);
		$remaining = array_diff($results, $result_category_keys);

		$result_section = array();

		if (!empty($remaining))
		{
			$remaining_ids = implode(',', $remaining);

			$querys = $_db->getQuery(true);
			$querys->select('distinct sp.entry_id,sp.ordering,sp.counter');

			$querys->from('#__xtdir_final_sectionpromoted as sp');
			$querys->from('#__sobipro_object as o');
			$querys->from('#__sobipro_relations as r');

			$querys->where('sp.entry_id in (' . $remaining_ids . ')');
			$querys->where('sp.entry_id = o.id');

			$querys->where('r.id = sp.entry_id');
			$querys->where('r.oType = \'entry\'');

			// $querys->where('section_id = ' . $_db->q($section_id));
			$querys->order('r.position');

			// $logger = XtdirLogger::getInstance();
			// $logger->log(JLog::INFO, '_getPromotedEntries querys: ' . $querys->__toString());

			$_db->setQuery($querys, 0, 0);
			$result_section = $_db->loadObjectList('entry_id');
		}

		$result = array();

		if (!empty($result_category))
		{
			foreach ($result_category as $entry_id => $row)
			{
				$result[$entry_id] = $row;
			}
		}

		if (!empty($result_section))
		{
			foreach ($result_section as $entry_id => $row)
			{
				$result[$entry_id] = $row;
			}
		}

		// $logger = XtdirLogger::getInstance();
		// $logger->log(JLog::INFO, 'querys: ' . $querys->__toString());

		return $result;
	}

	/**
	 * _getClusteredEntries
	 *
	 * @param   array  $results           Params
	 * @param   array  $ordering_entries  Params
	 *
	 * @return	array
	 */
	private static function _getClusteredEntries($results, $ordering_entries)
	{
		$entries = array();

		foreach ($ordering_entries as $entry_id => $object)
		{
			if (!array_key_exists($object->ordering, $entries))
			{
				$entries[$object->ordering] = array();
			}

			$entries[$object->ordering][] = $object;
		}

		ksort($entries);

		$sorttype_id = EParameter::getComponentParam('com_xtdir', 'sorttype_id', 'M');

		// Multi Armed Bandit
		if ($sorttype_id == 'M')
		{
			$final = array();

			foreach ($entries as $ordering => $clustered)
			{
				$clustered = self::_executeMultiArmedBanditAlgo($clustered);

				foreach ($clustered as $object)
				{
					$final[] = $object->entry_id;
				}
			}

			return $final;
		}
		else
		{
			$clustered_entries = array();

			// Default
			foreach ($entries as $ordering => $clustered)
			{
				foreach ($clustered as $object)
				{
					$clustered_entries[] = $object->entry_id;
				}
			}

			return $clustered_entries;
		}
	}

	/**
	 * _executeSorttype
	 *
	 * @param   array  &$entries  Params
	 * @param   int    $sorttype  Params
	 *
	 * @return	array
	 */
	private static function _executeSorttype(&$entries, $sorttype)
	{
		if ($sorttype == self::SORTTYPES_DEFAULT)
		{
			return $entries;
		}

		$manual_entries = array();
		$default_entries = array();

		foreach ($entries as $entry)
		{
			if (($entry->ordering == self::SECTION_PROMOTED) || ($entry->ordering == self::CATEGORY_PROMOTED))
			{
				$default_entries[] = $entry;
			}
			else
			{
				$manual_entries[] = $entry;
			}
		}

		if ($sorttype == self::SORTTYPES_RANDOM)
		{
			self::_executeRandomAlgo($manual_entries);
			self::_executeRandomAlgo($default_entries);
		}
		elseif ($sorttype == self::SORTTYPES_HITS)
		{
			self::_executeHitsAlgo($manual_entries);
			self::_executeHitsAlgo($default_entries);
		}
		elseif ($sorttype == self::SORTTYPES_MULTI_ARMED_BANDIT)
		{
			self::_executeMultiArmedBanditAlgo($manual_entries);
			self::_executeMultiArmedBanditAlgo($default_entries);
		}

		$final = array_merge($manual_entries, $default_entries);

		return $final;
	}

	/**
	 * _executeRandomAlgo
	 *
	 * @param   array  &$entries  Params
	 *
	 * @return	array
	 */
	private static function _executeRandomAlgo(&$entries)
	{
		return usort(
				$entries,
				function ($a, $b)
				{
					return rand(0, 1);
				}
		);
	}

	/**
	 * _executeHitsAlgo
	 *
	 * @param   array  $entries  Params
	 *
	 * @return	array
	 */
	private static function _executeHitsAlgo($entries)
	{
		usort(
			$entries,
			function ($a, $b)
			{
				if ($a->counter == $b->counter)
				{
					return 0;
				}

				return ($a->counter > $b->counter) ? -1 : 1;
			}
		);

		return $entries;
	}

	/**
	 * _executeMultiArmedBanditAlgo
	 *
	 * @param   array  $entries  Params
	 *
	 * @return	array
	 */
	private static function _executeMultiArmedBanditAlgo($entries)
	{
		$entries = self::_executeHitsAlgo($entries);

		$n = count($entries);
		$k = ceil(floatval($n) * 0.1);

		for ($i = 0; $i < $k; $i++)
		{
			$j = rand(0, $n - 1);
			$v = $entries[$j];
			array_splice($entries, $j, 1);
			array_unshift($entries, $v);
		}

		return $entries;
	}

	/**
	 * getExpiration
	 *
	 * @param   int  $entry_id     Params
	 * @param   int  $tiertype_id  Params
	 *
	 * @return	date
	 */
	public static function getExpiration($entry_id, $tiertype_id)
	{
		$promotionsModel = FOFModel::getTmpInstance('Promotions', 'XtdirModel');
		$promotionsModel->setState('entry_categories', $entry_id);
		$promotionsModel->setState('tiertype_id', $tiertype_id);
		$promotion = $promotionsModel->getFirstItem(true);

		if (isset($promotion->id))
		{
			$expiration = $promotion->expiration;
			$unit = $promotion->unit;
		}
		else
		{
			$promotionsModel = FOFModel::getTmpInstance('Promotions', 'XtdirModel');
			$promotionsModel->setState('entry_id', $entry_id);
			$promotionsModel->setState('tiertype_id', $tiertype_id);
			$promotion = $promotionsModel->getFirstItem(true);

			if (isset($promotion->id))
			{
				$expiration = $promotion->expiration;
				$unit = $promotion->unit;
			}
			else
			{
				return '0000-00-00 00:00:00';
			}
		}

		if (empty($expiration))
		{
			return '0000-00-00 00:00:00';
		}

		$now = JFactory::getDate()->toUnix();

		$date = new DateTime;
		$date->setTimestamp($now);

		// (D)AY, (M)ONTH, (Y)EAR
		$interval = 'P' . $expiration . strtoupper($unit);
		$date->add(new DateInterval($interval));
		$final_date = $date->getTimestamp();

		return JFactory::getDate($final_date)->toSql();
	}
}
