<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * Searcher.
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
*/
class Searcher
{
	private $_params = null;
	const ALL_ENTRIES = '----';

	/**
	 * Constructor.
	 *
	 * @param   array  &$params  Params
	 *
	 * @since	1.0
	 */
	public function __construct(&$params)
	{
		$this->_params = $params;

		$this->_search_username = $this->_params->get('search_username', 0);
		$this->_search_name = $this->_params->get('search_name', 0);
		$this->_custom_field_names = $this->_params->get('custom_field_names', null);
		$this->_search_category_list = $this->_params->get('search_category_list', null);
		$this->_desc_list = $this->_params->get('desc_list', null);
		$this->_menu_itemid = $this->_params->get('menu_itemid', null);
		$this->_ft_mode = $this->_params->get('ft_mode', 0);
		$this->_ft_mode_qe = $this->_params->get('ft_mode_qe', 0);
		$this->_sql_big_selects = $this->_params->get('sql_big_selects', 1);

		// $this->_params->get('order_query', null);
		$this->_order_query = null;

		$this->_randomize = $this->_params->get('randomize', 0);
		$this->_allow_empty = $this->_params->get('allow_empty', 1);
		$this->_allow_guest_entries = $this->_params->get('allow_guest_entries', 1);
		$this->_only_approved = $this->_params->get('only_approved', 0);
		$this->_native_sef = $this->_params->get('native_sef', 1);
		$this->_limit = $this->_params->get('search_limit', 50);
		$this->_section = $this->_params->get('title', 'Change the Title');

		$this->_cleanMultiSelect($this->_custom_field_names);
		$this->_cleanMultiSelect($this->_search_category_list);
		$this->_cleanMultiSelect($this->_desc_list);
		$this->_cleanMultiSelect($this->_order_query);

		define('PLG_SEARCH_XTDIRSOBIPRO_ITEMS', $this->_section);

		$this->_lang = Sobi::Lang();

		// To have the static methods available
		FOFModel::getTmpInstance('Fields', 'XtdirModel');
	}

	/**
	 * execute
	 *
	 * @param   string  $text      Text to search
	 * @param   string  $phrase    How to search
	 * @param   string  $ordering  How to order
	 *
	 * @return	array  Result - The sql must return the following fields that are used in a common display
	 * routine: href, title, section, created, text, browsernav
	 *
	 * @since	1.0
	 */
	public function execute($text, $phrase = '', $ordering = '')
	{
		$log_level = EParameter::getComponentParam(CXTDIR, 'log_level', 0);
		$log_mode = EParameter::getComponentParam(CXTDIR, 'log_mode', 0);

		$logger = XtdirLogger::getInstance($log_level, $log_mode);
		$catalog_list = EExtensionHelper::getExtensionParam('system', 'xtdir', 'catalog_list');

		$_db = JFactory::getDbo();

		// SEARCH TEXT
		$text = trim($text);

		if ($text == '')
		{
			$logger->log(JLog::INFO, 'No text');
		}

		// CATEGORIES
		$sid_list = null;

		if ((preg_match('/c:([0-9,]+)/', $text, $matches)) && (count($matches) == 2))
		{
			$sid_list = $matches[1];
			$text = preg_replace('/(\ *)c:([0-9,]+)/', '', $text);

			// Case, just a category filter
			if ($text == '')
			{
				// Return all entries, filtered by category
				$text = self::ALL_ENTRIES;
			}
		}

		$category_path = $this->_build_category_path($_db, $sid_list);
		$empty_search = ((strcmp($text, self::ALL_ENTRIES) == 0) && ($this->_allow_empty));

		if ($empty_search)
		{
			$text = '';
		}

		// EMPTY SEARCH MODE
		if ($empty_search)
		{
			$logger->log(JLog::INFO, 'EMPTY SEARCH MODE');
		}

		// Not allowed
		if ((!$this->_allow_empty) && (empty($text)))
		{
			return array();
		}

		// SECTIONS
		if (!empty($catalog_list))
		{
			$catalogs = $catalog_list;
			sort($catalogs);
			$logger->log(JLog::INFO, 'Catalogs = ' . implode('-', $catalogs));
		}
		else
		{
			$catalogs = SectionsHelper::findAll();
			sort($catalogs);
			$logger->log(JLog::INFO, 'All catalogs = ' . implode('-', $catalogs));
		}

		// FIELD NAME FOR EACH SECTION
		$field_name = $this->_getFieldNames($catalogs);

		if (count($this->_custom_field_names) == count($catalogs))
		{
			$field_name = $this->_getFields($catalogs, $this->_custom_field_names);
		}
		else
		{
			$logger->log(JLog::INFO, 'Not matching descriptions to catalogs.');
		}

		// DESCRIPTIONS
		$field_desc = array();

		if (count($this->_desc_list) == count($catalogs))
		{
			$field_desc = $this->_getFields($catalogs, $this->_desc_list);
			$logger->log(JLog::INFO, 'Desc Fields ', $field_desc);
		}
		else
		{
			$logger->log(JLog::INFO, 'No descs');
		}

		// RANDOMIZE
		if ($this->_randomize)
		{
			$order = 'RAND()';
		}

		if ($this->_sql_big_selects)
		{
			$_db->setQuery("SET OPTION SQL_BIG_SELECTS=1");
			$_db->execute();
		}

		// BUILD QUERY
		$i = 0;
		$rows = array();

		// FOR EACH SECTION
		foreach ($catalogs as $catalog_pid)
		{
			$this->_current_section = $catalog_pid;

			// FIELDS
			$searchFields = FOFModel::getTmpInstance('Fields', 'XtdirModel');
			$fields = $searchFields->getSearchFields($catalog_pid);

			if (empty($fields))
			{
				$logger->log(JLog::INFO, 'No search fields. Section=' . $catalog_pid);

				return $this->_fillRowData($rows, $field_name, $field_desc);
			}

			// IF NOT RANDOMIZE, SELECTED ORDER
			if (!$this->_randomize)
			{
				$order = $this->_genSqlOrder($ordering);
				$logger->log(JLog::INFO, 'Order:', $order);
			}

			// ALL ENTRIES UNDER THE SAME SECTION
			$catalog_path = 't.path LIKE ' . $_db->Quote($catalog_pid . '-%');

			$query = $_db->getQuery(true);
			$query->select('n.baseData AS title, a.updatedTime AS created, ' . 'a.id as slug, ' . 'c.id AS catslug, ' . '"-" AS section, "2" AS browsernav,' . 'a.id id,' . $_db->Quote($catalog_pid) . ' catalog_pid');
			$query->from('#__sobipro_object AS a');

			// SECTION
			$query->innerJoin('#__xtdir_trees AS t ON a.id = t.id AND ' . $catalog_path);

			// CATEGORY
			$query->innerJoin('#__sobipro_object AS c ON t.parent = c.id');

			if (($this->_search_username) || ($this->_search_name) || (!$this->_allow_guest_entries))
			{
				$query->innerJoin('#__users AS u ON u.id = a.owner');
			}

			// GET NAME FORM FIELD, NOT FROM OBJECT
			$query->innerJoin('#__sobipro_field_data AS n ON a.id = n.sid AND n.fid=' . $field_name[$catalog_pid]->fid);

			$query->where('a.oType=' . $_db->Quote('entry'));
			$query->where('c.oType=' . $_db->Quote('category'));
			$query->where('a.state=1 AND c.state=1');

			if ($this->_only_approved)
			{
				$query->where('a.approved=1');
			}

			if (!empty($category_path))
			{
				$query->where($category_path);
			}

			if (!$empty_search)
			{
				$this->_addSearchFieldsCondition($query, $fields, $phrase, $text);
			}

			$query->order($order);

			$logger->log(JLog::INFO, 'Query ' . $query);

			$_db->setQuery($query, 0, $this->_limit);

			try
			{
				$subrows = $_db->loadObjectList();

				if ($error = $_db->getErrorMsg())
				{
					$logger->log(JLog::ERROR, $error);
				}
			}
			catch (Exception $e)
			{
				$logger->log(JLog::ERROR, $e->getMessage());
			}

			$logger->log(JLog::INFO, 'Results: ' . count($subrows));

			if (!empty($subrows))
			{
				$rows = array_merge($rows, $subrows);
			}

			$i++;
		}

		if (!empty($rows))
		{
			return $this->_fillRowData($rows, $field_name, $field_desc);
		}
		else
		{
			$logger->log(JLog::INFO, 'No results');
		}

		return $rows;
	}

	/**
	 * _fillRowData
	 *
	 * @param   array  &$rows        Catalogs
	 * @param   array  &$field_name  Field names
	 * @param   array  &$field_desc  Field desc
	 *
	 * @return	array.
	 *
	 * @since	5.0
	 */
	private function _fillRowData(&$rows, &$field_name, &$field_desc)
	{
		// Menu item id param
		$this->_menu_itemid = trim($this->_menu_itemid);
		$theitemid = $this->_menu_itemid;

		if (is_numeric($this->_menu_itemid))
		{
			$this->_menu_itemid = '&Itemid=' . $this->_menu_itemid;
		}
		else
		{
			$this->_menu_itemid = '';
		}

		// Lang
		$lang = JRequest::get('lang');
		$lng = null;

		if (($lang) && (array_key_exists('lang', $lang)))
		{
			$lng = '&lang=' . $lang['lang'];
		}

		$cats_cache = array();
		$results_cache = array();
		$results = array();

		foreach ($rows as $key => $row)
		{
			// Cached categories
			if (array_key_exists($row->catslug, $cats_cache))
			{
				$c_name = $cats_cache[$row->catslug];
			}
			else
			{
				$c = SobiproHelper::loadAndExtend($row->catslug);
				$c_name = $c->get('name');
				$cats_cache[$row->catslug] = $c_name;
			}

			// It's in the results, just add the category to the previous result
			if (array_key_exists($row->slug, $results_cache))
			{
				$r = $results[$row->slug];
				$r->section .= ', ' . $c_name;
				$results[$row->slug] = $r;
				continue;
			}
			else
			{
				// This is new entry to be cached
				$results_cache[$row->slug] = $row->slug;
			}

			// Entry
			$entry = SobiproHelper::loadAndExtend($row->slug);

			// Name
			$field = $field_name[$row->catalog_pid];
			$name = $entry->get($field->nid);
			$nid = $entry->get('nid');
			$row->title = $name;

			// Category
			$row->section = $this->_section . ' / ' . $c_name;

			// Descriptions
			if (array_key_exists($row->catalog_pid, $field_desc))
			{
				$field = $field_desc[$row->catalog_pid];
				$row->text = $entry->get($field->nid);
			}
			else
			{
				$row->text = $c_name;
			}

			// Native Sef, or custom sef
			if ($this->_native_sef)
			{
				$urlparams = array(
								'pid' => $row->catalog_pid,
								'sid' => $row->slug,
								'title' => (Sobi::Cfg('sef.alias', true) ? $nid : $name)
				);
				$url = Sobi::Url($urlparams, false, false, false, false);

				if ($theitemid)
				{
					$pattern = '/(\&Itemid\=\d+)/i';
					$url = preg_replace($pattern, $this->_menu_itemid, $url);
				}

				$row->href = $url;
			}
			else
			{
				$url = 'index.php?option=com_sobipro&pid=' . $row->catalog_pid . '&sid=' . $row->slug . ':' . urlencode($name) . '&catid=' . $row->catslug . $this->_menu_itemid . $lng;
				$row->href = JRoute::_($url);
			}

			// Results buffer
			$results[$row->slug] = $row;
		}

		if (count($rows) > $this->_limit)
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::INFO, 'Hit limit!');
		}

		return array_values($results);
	}

	/**
	 * Sort Field Names
	 *
	 * @param   array  &$catalogs     Catalogs
	 * @param   array  &$field_names  Field names
	 *
	 * @return	array.
	 *
	 * @since	1.0
	 */
	private function _getFields(&$catalogs, &$field_names)
	{
		$descFields = FOFModel::getTmpInstance('Fields', 'XtdirModel');
		$descFields->setState('enabled', 1);
		$descFields->setState('sections', implode(',', $catalogs));
		$descFields->setState('ids', implode(',', $field_names));
		$descFields->setState('filter_order', 'section');

		$query = $descFields->buildQuery(true);

		$_db = JFactory::getDbo();
		$_db->setQuery($query);
		$results = $_db->loadObjectList('section');

		return $results;
	}

	/**
	 * Generate SQL for order
	 *
	 * @param   string  $ordering  param
	 *
	 * @return	array.
	 *
	 * @since	1.0
	 */
	private function _genSqlOrder($ordering)
	{
		switch ($ordering)
		{
			case 'alpha':
				$order = 'title ASC';
				break;

			case 'category':
				$order = 'c.name ASC, title ASC';
				break;

			case 'popular':
				$order = 'a.counter DESC, title ASC';
				break;

			case 'newest':
				$order = 'a.updatedTime DESC, title ASC';
				break;

			case 'oldest':
				$order = 'a.updatedTime ASC, title ASC';
				break;

			default:
				$order = 'title DESC';
		}

		return $order;
	}

	/**
	 * cleanListOfNumerics
	 *
	 * @param   string  $listOfNumerics  param
	 *
	 * @return	array.
	 *
	 * @since	1.0
	 */
	private function _cleanListOfNumerics($listOfNumerics)
	{
		return preg_replace('/[^,0-9]/', '', $listOfNumerics);
	}

	/**
	 * _build_category_path
	 *
	 * @param   object  &$_db      param
	 * @param   string  $sid_list  param
	 *
	 * @return	string.
	 *
	 * @since	1.0
	 */
	private function _build_category_path(&$_db, $sid_list)
	{
		if ($sid_list)
		{
			$sid_list = $this->_cleanListOfNumerics($sid_list);

			if ($sid_list)
			{
				$sid_list = explode(',', $sid_list);
			}
		}

		if (is_array($sid_list))
		{
			$category_list = $sid_list;
		}
		else
		{
			if (is_array($this->_search_category_list))
			{
				$category_list = $this->_search_category_list;
			}
		}

		$category_path = null;

		if ((($this->_search_category_list) || ($sid_list)) && (count($category_list) > 0))
		{
			// T.path LIKE '%-999-%' OR t.path LIKE '%-888-%'
			$category_path = array();

			for ($i = 0; $i < count($category_list); $i++)
			{
				$category = $category_list[$i];
				$category_path[] = 't.path LIKE ' . $_db->Quote('%-' . $category . '-%');
			}

			$category_path = '(' . join(' OR ', $category_path) . ')';
		}

		return $category_path;
	}

	/**
	 * _getFieldNames
	 *
	 * @param   array  $catalogs  param
	 *
	 * @return	array.
	 *
	 * @since	1.0
	 */
	private function _getFieldNames($catalogs)
	{
		$fields = FOFModel::getTmpInstance('Fields', 'XtdirModel');
		$fields->setState('enabled', 1);
		$fields->setState('sections', implode(',', $catalogs));
		$fields->setState('isNameField', 1);
		$fields->setState('filter_order', 'section');

		$query = $fields->buildQuery(true);

		$_db = JFactory::getDbo();
		$_db->setQuery($query);
		$results = $_db->loadObjectList('section');

		return $results;
	}

	/**
	 * _addSearchFieldsCondition
	 *
	 * @param   object  &$query   param
	 * @param   array   &$fields  param
	 * @param   string  $phrase   param
	 * @param   string  $text     param
	 *
	 * @return	array.
	 *
	 * @since	1.0
	 */
	private function _addSearchFieldsCondition(&$query, &$fields, $phrase, $text)
	{
		if (count($fields) == 0)
		{
			return;
		}

		if ($phrase == 'all')
		{
			$texts = explode(' ', $text);

			if (count($texts) == 1)
			{
				$cond = $this->_addSearchInboxConditionAnyExact($fields, $phrase, $text);
				$query->where($cond);
			}
			else
			{
				foreach ($texts as $word)
				{
					$cond = $this->_addSearchInboxConditionAnyExact($fields, $phrase, $word);
					$query->where($cond);
				}
			}
		}
		else
		{
			$cond = $this->_addSearchInboxConditionAnyExact($fields, $phrase, $text);
			$query->where($cond);
		}
	}

	/**
	 * _addSearchInboxConditionAnyExact
	 *
	 * @param   array   &$fields  param
	 * @param   string  $phrase   param
	 * @param   string  $text     param
	 *
	 * @return	array.
	 *
	 * @since	1.0
	 */
	private function _addSearchInboxConditionAnyExact(&$fields, $phrase, $text)
	{
		$conds = array();
		$conds[] = $this->_generateCondition('n.baseData', $phrase, $text);
		$conds[] = $this->_generateCondition('a.name', $phrase, $text);

		if ($this->_search_username)
		{
			$conds[] = $this->_generateCondition('u.username', $phrase, $text);
		}

		if ($this->_search_name)
		{
			$conds[] = $this->_generateCondition('u.name', $phrase, $text);
		}

		foreach ($fields as $field)
		{
			if (($field->fieldType == 'inbox') || ($field->fieldType == 'textarea'))
			{
				$conds[] = $this->_addSearchInboxCondition($field, $phrase, $text);
			}
			else
			{
				$conds[] = $this->_addSearchSelectCondition($field, $phrase, $text);
			}
		}

		$cond = '(' . join(' OR ', $conds) . ')';

		return $cond;
	}

	/**
	 * _addSearchInboxCondition
	 *
	 * @param   array   &$field  param
	 * @param   string  $phrase  param
	 * @param   string  $text    param
	 *
	 * @return	array.
	 *
	 * @since	1.0
	 */
	private function _addSearchInboxCondition(&$field, $phrase, $text)
	{
		$_db = JFactory::getDbo();
		$subquery = $_db->getQuery(true)->select('f.sid')->from('#__sobipro_field_data AS f')->where('f.fid = ' . $_db->Quote($field->fid))->where('f.section = ' . $_db->Quote($this->_current_section))->where('f.lang = ' . $_db->Quote($this->_lang))->where('f.sid = a.id')->where('f.copy = 0');

		$cond = $this->_generateCondition('f.baseData', $phrase, $text);
		$subquery->where($cond);

		return 'a.id IN (' . $subquery->__toString() . ')';
	}

	/**
	 * _addSearchSelectCondition
	 *
	 * @param   array   &$field  param
	 * @param   string  $phrase  param
	 * @param   string  $text    param
	 *
	 * @return	array.
	 *
	 * @since	1.0
	 */
	private function _addSearchSelectCondition(&$field, $phrase, $text)
	{
		$_db = JFactory::getDbo();
		$subquery = $_db->getQuery(true);
		$subquery->select('f.sid');
		$subquery->from('#__sobipro_field_option_selected AS f');
		$subquery->where('f.fid = ' . $_db->Quote($field->fid));
		$subquery->where('f.sid = a.id');
		$cond = $this->_generateCondition('f.optValue', $phrase, $text);
		$subquery->where('copy = 0');
		$subquery->where($cond);

		return 'a.id IN (' . $subquery->__toString() . ')';
	}

	/**
	 * _generateCondition
	 *
	 * @param   string  $fieldName  param
	 * @param   string  $phrase     param
	 * @param   string  $text       param
	 *
	 * @return	array.
	 *
	 * @since	1.0
	 */
	private function _generateCondition($fieldName, $phrase, $text)
	{
		switch ($phrase)
		{
			case 'any':
				$words = explode(' ', $text);
				$conds = array();

				foreach ($words as $word)
				{
					$conds[] = $this->_generateTextCondition($fieldName, $word);
				}

				return '(' . join(' OR ', $conds) . ')';
				break;

			case 'all':
			case 'exact':
			default:
				return $this->_generateTextCondition($fieldName, $text);
		}

		return null;
	}

	/**
	 * _generateTextCondition
	 *
	 * @param   object  $fieldName  param
	 * @param   string  $text       param
	 *
	 * @return	array.
	 *
	 * @since	1.0
	 */
	private function _generateTextCondition($fieldName, $text)
	{
		if ($this->_ft_mode)
		{
			$phrase = 'exact';
		}

		if ($this->_ft_mode_qe)
		{
			$qe_query = 'WITH QUERY EXPANSION';
		}
		else
		{
			$qe_query = '';
		}

		$_db = JFactory::getDbo();

		if ($this->_ft_mode)
		{
			$text = $_db->Quote($_db->escape($text, true), false);

			return 'MATCH(' . $fieldName . ') AGAINST (' . $text . $qe_query . ')';
		}
		else
		{
			$text = '%' . $_db->escape($text, true) . '%';
			$text = $_db->Quote($text);

			return $fieldName . ' LIKE ' . $text;
		}

		return null;
	}

	/**
	 * _cleanMultiSelect
	 *
	 * @param   array  &$options  param
	 *
	 * @return	void
	 */
	private function _cleanMultiSelect(&$options)
	{
		if (($options) && (is_array($options)))
		{
			$options = array_filter($options, 'is_numeric');
		}
	}
}
