<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * ESystemHelper.
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
 */
class ESystemHelper
{
	const MAX_COLUMNS = 7;

	/**
	 * loadTotals.
	 *
	 * @param   object  $view  Param
	 *
	 * @return	void
	 */
	public static function loadTotals($view)
	{
		$treeModel = FOFModel::getTmpInstance('Trees', 'XtdirModel');
		$total = $treeModel->getTotal();
		$view->assign('total', $total);

		$logger = XtdirLogger::getInstance();
		$logger->log(JLog::INFO, 'loadTotals - Trees: ' . $total);

		if ($total > 0)
		{
			$treeModel->setState('filter_order_Dir', 'DESC');
			$treeModel->setState('limit', 1);
			$items = $treeModel->getItemList();

			$item = $items[0];
			$view->assign('last_update', $item->last_update);

			$logger->log(JLog::INFO, 'loadTotals - last_update: ' . $item->last_update);
		}

		$objects = FOFModel::getTmpInstance('Objects', 'XtdirModel');
		$objects->setState('oType', 'entry');
		$objects->setState('promotype_id', ord('M'));
		$total = $objects->getTotal();
		$view->assign('total_promoentries', $total);

		$logger->log(JLog::INFO, 'loadTotals - total_promoentries: ' . $total);

		$finalPromotedEntries = FOFModel::getTmpInstance('FinalCategoryPromoted', 'XtdirModel');
		$total = $finalPromotedEntries->getTotal();
		$view->assign('total_category_promotion', $total);

		$logger->log(JLog::INFO, 'loadTotals - total_category_promotion: ' . $total);

		$finalPromotedEntries = FOFModel::getTmpInstance('FinalSectionPromoted', 'XtdirModel');
		$total = $finalPromotedEntries->getTotal();
		$view->assign('total_section_promotion', $total);

		$logger->log(JLog::INFO, 'loadTotals - total_section_promotion: ' . $total);

		// Cache status
		$config = JFactory::getConfig();
		$caching = $config->get('caching');
		$caching_label = $caching ? 'JENABLED' : 'JDISABLED';
		$view->assign('cache_label', $caching_label);

		$logger->log(JLog::INFO, 'loadTotals - cache_label: ' . $caching_label);

		if ($caching)
		{
			$caching_class = 'label-success';
		}
		else
		{
			$caching_class = 'label-warning';
		}

		$view->assign('cache_class', $caching_class);

		$logger->log(JLog::INFO, 'loadTotals - cache_class: ' . $caching_class);

		$cachetime = $config->get('cachetime');
		$view->assign('cache_time', $cachetime);

		$logger->log(JLog::INFO, 'loadTotals - cache_time: ' . $cachetime);

		$lang = JFactory::getLanguage();
		$option = 'com_config';
		$lang->load($option, JPATH_BASE, null, false, false) || $lang->load($option, JPATH_BASE . "/components/$option", null, false, false) || $lang->load($option, JPATH_BASE, $lang->getDefault(), false, false) || $lang->load($option, JPATH_BASE . "/components/$option", $lang->getDefault(), false, false);
	}

	/**
	 * generateWidget.
	 *
	 * @return	bool
	 */
	public static function generateWidget()
	{
		$sections = EExtensionHelper::getExtensionParam('system', 'xtdir', 'catalog_list');

		if (empty($sections))
		{
			$sections = SectionsHelper::findAll();
		}

		$input = array(
			'savestate'	=> 0,
			'task'		=> 'browse',
			'caching'	=> false,

			// Params
			'chart_id' => 999,

			'sections' => $sections,
			'start_level' => 1,
			'end_level' => 1,
			'count' => 99,
			'show_total_entries' => 1,
			'show_total_catsentries' => 0,
			'legend_position' => 'none',

			// Counter Desc
			'query_order' => '3',
			'require_stats' => true,

			'current_sid' => null,
			'my_itemid' => null,
			'sub_Itemids' => null,
			'default_Itemid' => null,

			// Entries
			'counter_type' => 2,
			'hide_empty' => true
		);

		$config = array(
						'option'	=> 'com_xtdir',
						'view'		=> 'statswidget',
						'layout'	=> 'pie',
						'input'		=> $input
		);

		ob_start();
		FOFDispatcher::getTmpInstance('com_xtdir', 'statswidget', $config)->dispatch();
		$result = ob_get_contents();
		ob_end_clean();

		return $result;
	}

	/**
	 * loadAjaxIndexer
	 *
	 * @param   object  $view  Param
	 *
	 * @return	void
	 */
	public static function loadAjaxIndexer($view)
	{
		$ajax_index = EExtensionHelper::getExtensionParam('system', 'xtdir', 'ajax_index', 1);
		$view->assign('ajax_index', $ajax_index);

		if ($ajax_index)
		{
			$file = EHtml::getRelativeFile('js', 'com_xtdir/index.min.js');

			if ($file)
			{
				$dependencies = array();
				$dependencies['index'] = array('extlycore');

				Extly::initApp(CXTDIR_VERSION, $file, $dependencies);
			}
		}
		else
		{
			Extly::initApp(CXTDIR_VERSION);
		}
	}
}
