<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * XtdirModelPromotions
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 * @since       1.0
*/
class XtdirModelPromotions extends FOFModel
{
	/**
	 * buildQuery
	 *
	 * @param   bool  $overrideLimits  Param
	 *
	 * @return	FOFQuery
	 */
	public function buildQuery($overrideLimits = false)
	{
		$db = $this->getDbo();

		$query = $db->getQuery(true)->select('p.*, s.name as section_name, c.name as category_name')->from($db->quoteName('#__xtdir_promotions', 'p'))->join('INNER', $db->quoteName('#__sobipro_object', 's') . ' ON p.section_id = s.id')->join('LEFT OUTER', $db->quoteName('#__sobipro_object', 'c') . ' ON p.category_id = c.id');

		$fltId = $this->getState('id', null, 'int');

		if ($fltId != '')
		{
			$query->where($db->qn('p.id') . ' = ' . $db->q($fltId));
		}

		$fltSearch = $this->getState('search', null, 'string');

		if ($fltSearch != '')
		{
			$query->where('(p.' . $db->qn('name') . ' LIKE ' . $db->q('%' . $fltSearch . '%') . ' OR ' . 's.' . $db->qn('name') . ' LIKE ' . $db->q('%' . $fltSearch . '%') . ' OR ' . 'c.' . $db->qn('name') . ' LIKE ' . $db->q('%' . $fltSearch . '%') . ')');
		}

		$fltPromotypeId = $this->getState('promotype_id');

		if ($fltPromotypeId)
		{
			$query->where($db->qn('p.promotype_id') . ' = ' . $db->q($fltPromotypeId));
		}

		$fltTiertypeId = $this->getState('tiertype_id');

		if ($fltTiertypeId)
		{
			$query->where($db->qn('p.tiertype_id') . ' = ' . $db->q($fltTiertypeId));
		}

		$fltEntryCategories = $this->getState('entry_categories');

		if ($fltEntryCategories)
		{
			$subquery = $db->getQuery(true)->select('r.pid')->from($db->quoteName('#__sobipro_relations') . ' AS r')->where('r.id = ' . $db->q($fltEntryCategories));
			$query->where($db->qn('p.category_id') . ' IN (' . $subquery->__toString() . ')');
		}

		$fltEntryId = $this->getState('entry_id');

		if ($fltEntryId)
		{
			$subquery = $db->getQuery(true)->select('t.section')->from($db->quoteName('#__xtdir_trees') . ' AS t')->where('t.id = ' . $db->q($fltEntryId));
			$query->where($db->qn('p.section_id') . ' IN (' . $subquery->__toString() . ')');
		}

		$fltPublished = $this->getState('published', 1, 'cmd');

		if ($fltPublished != '')
		{
			$query->where('p.' . $db->qn('published') . ' = ' . $db->q($fltPublished));
		}

		$order = $this->getState('filter_order', 'p.id', 'cmd');

		if (!in_array($order, array_keys($this->getTable()->getData())))
		{
			$order = 'p.id';
		}

		$dir = $this->getState('filter_order_Dir', 'ASC', 'cmd');
		$query->order($order . ' ' . $dir);

		return $query;
	}

	/**
	 * This method runs before the $data is saved to the $table. Return false to
	 * stop saving.
	 *
	 * @param   array     &$data   The data to save
	 * @param   FOFTable  &$table  The table to save the data to
	 *
	 * @return  boolean  Return false to prevent saving, true to allow it
	 */
	protected function onBeforeSave(&$data, &$table)
	{
		if (is_array($data))
		{
			FOFModel::getTmpInstance('PromoTypes', 'XtdirModel');

			if ($data['promotype_id'] == XtdirModelPromoTypes::PROMOTYPES_SECTION)
			{
				$data['category_id'] = 0;
			}

			FOFModel::getTmpInstance('TierTypes', 'XtdirModel');

			if ($data['tiertype_id'] != XtdirModelTierTypes::TIERTYPES_DINAMIC)
			{
				$data['field_id'] = 0;
				$data['field_value'] = '';
			}
		}

		return parent::onBeforeSave($data, $table);
	}
}
