<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * XtdirToolbar
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
*/
class XtdirToolbar extends FOFToolbar
{
	/**
	 * getMyViews.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	protected function getMyViews()
	{
		return array();
	}

	/**
	 * onCpanelBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onCpanelsBrowse()
	{
		parent::onCpanelsBrowse();

		JToolBarHelper::title(CXTDIR_NAME . ': ' . JText::_('COM_XTDIR_TITLE_CPANELS'), 'xtdir-logo.png');
	}

	/**
	 * onSystemsEdit.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onSystemsEdit()
	{
		// On frontend, buttons must be added specifically
		list($isCli, $isAdmin) = FOFDispatcher::isCliAdmin();

		if (!$isAdmin && !$this->renderFrontendButtons)
		{
			return;
		}

		$option = $this->input->getCmd('option', CXTDIR);

		// Set toolbar title
		$subtitle_key = strtoupper($option . '_TITLE_' . FOFInflector::pluralize($this->input->getCmd('view', 'cpanel'))) . '_EDIT';
		JToolBarHelper::title(CXTDIR_NAME . ': ' . JText::_($subtitle_key), 'xtdir');

		// Set toolbar icons
		JToolBarHelper::apply();
		JToolBarHelper::save();
		JToolBarHelper::custom('index', 'process.png', 'process.png', 'COM_XTDIR_COMMON_INDEX_LABEL', false);
		JToolBarHelper::cancel();
	}

	/**
	 * onSearchsEdit.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onSearchesEdit()
	{
		$this->_simpleAddToolbar();
	}

	/**
	 * onSearchsEdit.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onFindersEdit()
	{
		$this->_simpleAddToolbar();
	}

	/**
	 * onPromoentriescfgsEdit.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onPromoentriescfgsEdit()
	{
		// On frontend, buttons must be added specifically
		list($isCli, $isAdmin) = FOFDispatcher::isCliAdmin();

		if (!$isAdmin && !$this->renderFrontendButtons)
		{
			return;
		}

		$option = $this->input->getCmd('option', 'com_xtdir');

		// Set toolbar title
		$subtitle_key = strtoupper($option . '_TITLE_' . FOFInflector::pluralize($this->input->getCmd('view', 'cpanel'))) . '_EDIT';
		JToolBarHelper::title(CXTDIR_NAME . ': ' . JText::_($subtitle_key), 'xtdir');

		// Set toolbar icons
		JToolBarHelper::apply();
		JToolBarHelper::save();
		JToolBarHelper::custom('index', 'process.png', 'process.png', 'COM_XTDIR_COMMON_INDEX_LABEL', false);
		JToolBarHelper::cancel();
	}

	/**
	 * onPagesCfgsEdit.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onPagesCfgsEdit()
	{
		$this->_simpleAddToolbar();
	}

	/**
	 * onObjectsBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onObjectsBrowse()
	{
		// Parent::onBrowse();

		$subtitle_key = strtoupper(CXTDIR . '_TITLE_' . $this->input->getCmd('view', 'cpanel'));
		JToolBarHelper::title(CXTDIR_NAME . ': ' . JText::_($subtitle_key), 'xtdir-logo.png');

		JToolBarHelper::custom('index', 'process.png', 'process.png', 'COM_XTDIR_COMMON_INDEX_LABEL', false);
	}

	/**
	 * onBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onBrowse()
	{
		parent::onBrowse();

		$subtitle_key = strtoupper(CXTDIR . '_TITLE_' . $this->input->getCmd('view', 'cpanel'));
		JToolBarHelper::title(CXTDIR_NAME . ': ' . JText::_($subtitle_key), 'xtdir-logo.png');
	}

	/**
	 * onPromotionsBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onPromotionsBrowse()
	{
		parent::onBrowse();

		$subtitle_key = strtoupper(CXTDIR . '_TITLE_' . $this->input->getCmd('view', 'cpanel'));
		JToolBarHelper::title(CXTDIR_NAME . ': ' . JText::_($subtitle_key), 'xtdir-logo.png');

		JToolBarHelper::custom('index', 'process.png', 'process.png', 'COM_XTDIR_COMMON_INDEX_LABEL', false);
	}

	/**
	 * onAdd.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onAdd()
	{
		parent::onAdd();

		$subtitle_key = strtoupper(CXTDIR . '_TITLE_' . $this->input->getCmd('view', 'cpanel'));
		JToolBarHelper::title(CXTDIR_NAME . ': ' . JText::_($subtitle_key), 'xtdir-logo.png');
	}

	/**
	 * onEdit.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onEdit()
	{
		parent::onEdit();

		$subtitle_key = strtoupper(CXTDIR . '_TITLE_' . $this->input->getCmd('view', 'cpanel'));
		JToolBarHelper::title(CXTDIR_NAME . ': ' . JText::_($subtitle_key), 'xtdir-logo.png');
	}

	/**
	 * onSimpleAdd.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	protected function _simpleAddToolbar()
	{
		// On frontend, buttons must be added specifically
		list($isCli, $isAdmin) = FOFDispatcher::isCliAdmin();

		if (!$isAdmin && !$this->renderFrontendButtons)
		{
			return;
		}

		$option = $this->input->getCmd('option', 'com_xtdir');

		// Set toolbar title
		$subtitle_key = strtoupper($option . '_TITLE_' . FOFInflector::pluralize($this->input->getCmd('view', 'cpanel'))) . '_EDIT';
		JToolBarHelper::title(CXTDIR_NAME . ': ' . JText::_($subtitle_key), 'xtdir');

		// Set toolbar icons
		JToolBarHelper::apply();
		JToolBarHelper::save();
		JToolBarHelper::cancel();
	}

	/**
	 * onPromoentriesEdit.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onPromoentriesEdit()
	{
		// On frontend, buttons must be added specifically
		list($isCli, $isAdmin) = FOFDispatcher::isCliAdmin();

		if (!$isAdmin && !$this->renderFrontendButtons)
		{
			return;
		}

		$option = $this->input->getCmd('option', CXTDIR);

		// Set toolbar title
		$subtitle_key = strtoupper($option . '_TITLE_' . FOFInflector::pluralize($this->input->getCmd('view', 'cpanel'))) . '_EDIT';
		JToolBarHelper::title(CXTDIR_NAME . ': ' . JText::_($subtitle_key), 'xtdir');

		// Set toolbar icons
		JToolBarHelper::apply();
		JToolBarHelper::save();
		JToolBarHelper::cancel();
	}
}
