<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/*
 * Based on AkeebaSubs - Nicholas K. Dionysopoulos *
 */

/*
 * /index.php?option=com_xtdir&view=callback&paymentmethod=paypal
 */

/**
 * XtdirControllerCallbacks
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 * @since       1.0
 */
class XtdirControllerCallbacks extends FOFController
{
	/**
	 * __construct
	 *
	 * @param   array  $config  Param
	 *
	 * @since   11.1
	 */
	public function __construct($config = array())
	{
		parent::__construct($config);

		$this->setThisModelName('XtdirModelOrders');
		$this->csrfProtection = false;
		$this->cacheableTasks = array();
	}

	/**
	 * Execute a task by triggering a method in the derived class.
	 *
	 * @param   string  $task  The task to perform. If no matching task is found, the '__default' task is executed, if defined.
	 *
	 * @return  mixed   The value returned by the called method, false in error case.
	 *
	 * @since   11.1
	 */
	public function execute($task)
	{
		$task = 'read';
		$this->input->set('task', 'read');
		parent::execute($task);
	}

	/**
	 * Execute a task by triggering a method in the derived class.
	 *
	 * @param   bool  $cachable  Param
	 *
	 * @return  void
	 */
	public function read($cachable = false)
	{
		$rawDataPost = JRequest::get('POST', 2);
		$rawDataGet = JRequest::get('GET', 2);
		$data = array_merge($rawDataGet, $rawDataPost);
		$paymentmethod = $this->input->getCmd('paymentmethod', 'none');

		$result = $this->runCallback($paymentmethod, $data);
		echo $result ? 'OK' : 'FAILED';
		JFactory::getApplication()->close();
	}

	/**
	 * onBeforeRead
	 *
	 * @return  bool
	 */
	public function onBeforeRead()
	{
		return true;
	}

	/**
	 * Runs a payment callback
	 *
	 * @param   string  $paymentmethod  Params
	 * @param   array   &$data          Params
	 *
	 * @return  bool
	 */
	public function runCallback($paymentmethod, &$data)
	{
		$dummy = $this->_getPaymentPlugins();

		$app = JFactory::getApplication();
		$jResponse = $app->triggerEvent(
				'onPaymentCallback',
				array(
						$paymentmethod,
						$data
				)
		);

		if (empty($jResponse))
		{
			return false;
		}

		$status = false;

		foreach ($jResponse as $response)
		{
			$status = $status || $response;
		}

		return $status;
	}

	/**
	 * Gets a list of payment plugins and their titles
	 *
	 * @return  bool
	 */
	private function _getPaymentPlugins()
	{
		JLoader::import('joomla.plugin.helper');
		JPluginHelper::importPlugin('xtdpayment');
		$app = JFactory::getApplication();
		$jResponse = $app->triggerEvent('onPaymentGetIdentity');

		$ret = array();

		foreach ($jResponse as $item)
		{
			if (is_object($item))
			{
				$ret[] = $item;
			}
			elseif (is_array($item))
			{
				if (array_key_exists('name', $item))
				{
					$ret[] = (object) $item;
				}
				else
				{
					foreach ($item as $anItem)
					{
						if (is_object($anItem))
						{
							$ret[] = $anItem;
						}
						else
						{
							$ret[] = (object) $anItem;
						}
					}
				}
			}
		}

		// Params: name, title
		return $ret;
	}
}
