/**
 * @package Extly.Components
 * @subpackage com_xtdir - Extended Directory for SobiPro
 *
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

/* jslint plusplus: true, browser: true, sloppy: true */
/* global jQuery, Request, Joomla, alert, Backbone */

define('objects', [ 'extlycore' ], function(Core) {
	"use strict";

	var $button = jQuery('.xt-ajax-button');
	$button.xtajaxbutton(function(response, currentTarget) {
		var $button = jQuery(currentTarget),
			$parent = $button.parent(),
			$tr = $parent.parent(),
			$toActivate = $parent.children('.hide'),
			$toHide = $parent.children('.active');

		$toActivate.removeClass('hide').addClass('active');
		$toHide.removeClass('active').addClass('hide');

		// Ordering
		$toActivate = $tr.find('span.inactive');
		$toHide = $tr.find('span.iactive');
		$toActivate.removeClass('inactive').addClass('iactive').attr('style', 'cursor: move;');
		$toHide.removeClass('iactive').addClass('inactive').attr('style', '');
	});

	var PromotedEntry = Core.ExtlyModel.extend({

		url : function() {
			return 'index.php?option=com_xtdir&view=objects&task=entryTogglePromoted';
		}

	});

	var PromotionView = Backbone.View.extend({

		events : {
			'click .xt-promoted-button' : 'onTogglePromoted'
		},

		initialize : function() {
			this.resetOrdering();
		},

		resetOrdering: function() {
			var td, tr;

			td = this.$('a.xt-promoted-e').filter('.hide').parent();
			tr = td.parent();
			tr.find('.input-ordering').addClass('disabled').attr('disabled','disabled');
		},

		onTogglePromoted: function(e) {
			var currentTarget = this.$(e.currentTarget),
				entryId = currentTarget.attr('entry_id'),
				token = currentTarget.attr('token'),
				model = new PromotedEntry();

			this.$(".loaderspinner").addClass('loading');
			this.currentEntry = currentTarget;

			model.on('change', this.loadResponse, this);
			model.fetch({
				data : {
					entryId : entryId,
					_token : token
				},

				wait : true,
				dataType : 'text',
				error : function(model, fail, xhr) {
					alert(fail.responseText);
				}
			});
		},

		loadResponse: function(message) {
			var finalpromotedentry, tr, td, ordering, promotype_id, promoentry_id, url;

			this.$(".loaderspinner").removeClass('loading');
			if (message.get('status')) {
				finalpromotedentry = message.get('finalpromotedentry');

				// Column
				td = this.currentEntry.parent();

				// Row
				tr = td.parent();

				// Hide all action buttons
				td.children('a').addClass('hide');

				// Activate the right action button according the new promoted status
				promotype_id = finalpromotedentry.promotype_id;
				if ((promotype_id === 'C') || (promotype_id === 'S'))
				{
					td.children('.xt-promoted-cs').removeClass('hide');
					tr.find('.input-ordering').addClass('disabled').attr('disabled','disabled');
				}
				else if (promotype_id === 'E')
				{
					td.children('.xt-promoted-e').removeClass('hide');

					promoentry_id = finalpromotedentry.promoentry_id;
					url = 'index.php?option=com_xtdir&view=promoentries&task=edit&id=' + promoentry_id;
					td.children('.xt-promoted-edit').attr('href', url);
					
					tr.find('.input-ordering').removeClass('disabled').removeAttr('disabled');
				}
				else
				{
					td.children('.xt-promoted-not').removeClass('hide');
					tr.find('.input-ordering').addClass('disabled').attr('disabled','disabled');
				}

				// New Ordering
				ordering = tr.find('.text_area');
				ordering.val(finalpromotedentry.ordering);
			} else {
				alert(message.get('error_message'));
			}
		}

	});

	var promotionView = new PromotionView({
		el : jQuery('#adminForm')
	});

});
