/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A.
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */

/* jslint plusplus: true, browser: true, sloppy: true */
/* global jQuery, Request, Joomla, alert, Backbone */

define('promotion', [ 'extlycore' ], function(Core) {
	"use strict";
	
	var Category = Core.ExtlyModel.extend({
		
		url : function() {
			return 'index.php?option=com_xtdir&view=promotions&task=getCategories';
		}

	});
	
	var Field = Core.ExtlyModel.extend({
		
		url : function() {
			return 'index.php?option=com_xtdir&view=promotions&task=getFields';
		}

	});

	var PromotionView = Backbone.View.extend({

		events : {
			'change #promotype_id' : 'onChangePromotype',
			'change #section_id' : 'onChangePromotype',
			'change #tiertype_id' : 'onChangeTiertype'			
		},

		initialize : function() {
			// Promotype
			var promotypeId = this.$('#promotype_id').val(),
				tiertypeId = this.$('#tiertype_id').val();
			// Category
			if (promotypeId === 'C') {
				this.$('.category-selection').fadeIn();
			} else {
			// Section
				this.$('.category-selection').fadeOut();
			}
			
			// Tiertype
			if (tiertypeId === 'D') {
				this.$('.dynamic-field').fadeIn();
			} else {
				this.$('.dynamic-field').fadeOut();
			}
			if (tiertypeId === 'P') {
				this.$('.expiration-group').fadeIn();
			} else {
				this.$('.expiration-group').fadeOut();
			}
		},

		onChangePromotype: function() {
			var promotypeId = this.$('#promotype_id').val();

			// Category
			if (promotypeId === 'C') {
				this.$('.category-selection').fadeIn();
				this.loadCategories();
			} else {
			// Section
				this.$('.category-selection').fadeOut();
			}
		},
		
		onChangeTiertype: function() {
			var tiertypeId = this.$('#tiertype_id').val();
			
			if (tiertypeId === 'D') {
				this.$('.dynamic-field').fadeIn();
				this.loadFields();
			} else {
				this.$('.dynamic-field').fadeOut();
			}
			if (tiertypeId === 'P') {
				this.$('.expiration-group').fadeIn();
			} else {
				this.$('.expiration-group').fadeOut();
			}
		},
		
		loadCategories: function() {
			var list = this.$('#category_id'),
				sectionId = this.$('#section_id').val(),
				model = new Category(),
				token = this.$('#token').attr('name');
			
			this.loaderList = list;
			Core.UiHelper.listReset(list);
			this.$(".loaderspinner").addClass('loading');
			
			model.on('change', this.loadItems, this);
			model.fetch({
				data : {
					section_id : sectionId,
					_token : token
				},
				
				wait : true,
				dataType : 'text',
				error : function(model, fail, xhr) {
					alert(fail.responseText);
				}
			});			
			
		},
		
		loadFields: function() {
			var list = this.$('#field_id'),
			    sectionId = this.$('#section_id').val(),
				model = new Field(),
				token = this.$('#token').attr('name');
			
			this.loaderList = list;
			Core.UiHelper.listReset(list);
			this.$(".loaderspinner").addClass('loading');
			
			model.on('change', this.loadItems, this);
			model.fetch({
				data : {
					section_id : sectionId,
					_token : token
				},
				
				wait : true,
				dataType : 'text',
				error : function(model, fail, xhr) {
					alert(fail.responseText);
				}
			});			
		},
		
		loadItems: function(message) {
			var list = this.loaderList, items;
			
			this.$(".loaderspinner").removeClass('loading');

			list.empty();
			if (message.get('status')) {
				items = message.get('items');
				_.each(items, function(item) {
					var opt = new Option();
					opt.value = item.id;
					opt.text = item.name;
					list.append(opt);						
				});
				list.trigger('liszt:updated');
			} else {
				alert(message.get('error_message'));
			}		
		}

	}),

	promosectionView = new PromotionView({
		el : jQuery('#adminForm')
	});

});
