/**
 * @package Extly.Modules
 * @subpackage xtdcategoriesfilterapp
 * 
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

/* jslint plusplus: true, browser: true, sloppy: true */
/* global jQuery, Request, Joomla, alert, Backbone */

/*

 TEST CASES:
 http://demo.xtdir.com/search-for-sobipro-methods/sobipro-search-with-chained-categories.html
 
 SobiPro - Form search (base cases to compare with the module)
 ---------------------

 Case 0	 Bookstore							=> 	Found 1 result(s)! Sun Bookstore
 Case 0	 Vivamus							=> 	Found 1 result(s)! Sun Bookstore 
 Case 1	 Blank - City Duckburg				=> 	Found 3 result(s)!
 Case 2	 * - City Duckburg					=> 	Found 3 result(s)!
 Case 3	 * - Category Communications 		=>  Found 225 result(s)!
 Case 4	 Blank - Category Communications	=> 	Found 0 result(s)!

 XTDir Chained Categories Control for Search App 	
 --------------------------------- 
 
 Case 0	 Bookstore							=> 	Found 1 result(s)! Sun Bookstore
 Case 0	 Bookstore / 2nd Step				=> 	Found 1 result(s)! Sun Bookstore
 Case 0	 Vivamus							=> 	Found 661 result(s)! 
 Case 1	 Blank - City Duckburg				=> 	Found 3 result(s)!	TEST OK
 Case 2	 Reset - City Duckburg				=> 	Found 3 result(s)!	TEST OK
 Case 3	 Manually adding *  - City Duckburg	=> 	Found 3 result(s)!	TEST OK
 Case 4	 Default search *  - Category Communications => Found 225 result(s)! TEST OK
 Case 5	 No Default search (Blank)- Category Communications	=> 	Found 0 result(s)!
 Case 6  Blank								=> Found 2011 result(s)!
 Case 7  Vivamus - City Duckburg / 2nd Step	=> Found 1 result(s)!

*/

define('xtdcategoriesfilterapp', [ 'extlycore', 'xtdhelper'], 
	function (Core, XtdHelper) {
	"use strict";
	
	var CatFilterAppView = Backbone.View.extend({
		
		events : {
			'change .sid_list' : 'onChange',
			'click [type="submit"]' : 'onSearchSubmit',
		},

		initialize : function() {
			var parent_div;

			this.$form = this.$('form');
			
			XtdHelper.UiHelper.initChainedCategories(this, this.$form.attr("id"));
			
			if (this.$form.xtautosave) {
				this.$form.xtautosave();
			}
			
			// Restore chained category tree according he hidden category_id ???
			
			// Category Input / div / div.controls / div.control-group
			parent_div = this.$('#field_select_category').parent().parent().parent();
			if (parent_div.hasClass('control-group'))
			{
				parent_div.hide();
			}
			else
			{			
				parent_div = this.$('#field_select_category').parent().parent();
				if (parent_div.hasClass('SPSearchCell'))
				{
					parent_div.hide();
				}					
			}
			
			// Disable submit button default click
			this.$('.btn-success').click(function(e) {
				e.preventDefault();
			});		
			
			this.$form = this.$('form');
		},
		
		onChange: function(e) {
			var $el = this.$(e.currentTarget), v;
			
			if (e.originalEvent) {
				v = $el.val();
				this.assignCategory(v);
			}				
		},
		
		assignCategory: function(v) {
			var f;
			
			v = v.replace('spc', '');
			
			f = this.$('#field_select_category');
			
			if (f.size() == 0)
			{
				alert('Warning: Chained Categories control is shown, but "field_select_category" is not defined in your section.');
			} else {
				f.val(v);
			}			
		},
		
		onSearchSubmit: function() {
			XtdHelper.UiHelper.onSearchSubmit(this, this.$form);
		}
		
	});
	
	// Just to be sure
	jQuery(document).ready(function() {
		var forms = jQuery('.SobiPro');
		_.each(forms, function(form)
				{
					var catFilterAppView = new CatFilterAppView({el : form});
				});
	});

});
