/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A.
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */

/* jslint plusplus: true, browser: true, sloppy: true */
/* global jQuery, Request, Joomla, alert, Backbone */

define('xtdhelper', [], function() {

	var UiHelper = {

		initChainedCategories : function(view, formId) {
			var levelp, classp, parent, classc, child;

			levelp = 1;
			classp = '.sid_list' + levelp;
			
			if (formId) {
				classp = '#' + formId + ' ' + classp;
			}
			
			parent = view.$(classp);
			while (parent.length !== 0) {
				classc = '.sid_list' + (levelp + 1);
				child = view.$(classc);

				if (child.length !== 0) {
					view.$(classc).chained(classp);
				}

				parent = child;
				classp = classc;
				levelp++;
			}
		},
		
		onSearchSubmit: function(view, form) {
			var spsearchbox, default_query, query, fieldsEmpty;
			
			spsearchbox = view.$('.XTSearchBox,#SPSearchBox');
			query = spsearchbox.val();
			default_query = view.$('.default_query').val();
			
			// If there's no default_query query, just submit - no change is possible
			if ( (_.isEmpty(default_query)) ||

					// There's a default query	and a search keyword					
			    ((query != window.xtSpSearchDefStr) && (!_.isEmpty(query))) )
			{
				// If searchboxes have different values, apply where really matters
				view.$('input[name=sp_search_for]').val(query);
				form.submit();
				return true;			
			}
			
			// (query == spSearchDefStr) || (_.isEmpty(query) ... just in case
			spsearchbox.val('');

			/*
			 * Complex Case, if empty && search fields (-categories) also empty => default_query
			*/			
			fieldsEmpty = UiHelper.searchFieldsEmpty(view, form); 
			if (fieldsEmpty) {						
				// Apply default_query
				view.$('input[name=sp_search_for]').val(default_query);
			}						
			
			form.submit();
			return true;
		},
		
		searchFieldsEmpty: function(view, $form) {
			var fields1, fields2, fields, isEmpty, f, v;
			
			fields1 = $form.find('input:not([name=sp_search_for],[name=xt_search_for],[type=password],[type=submit],[type=button],[type=hidden],[name=mj_rs_center_selector])').toArray();
			fields2 = $form.find('select:not([name=field_select_category],[name="field_select_category[]"],[name^=to_sid_list],[name=sid_list],[name=mj_rs_radius_selector])').toArray();
			fields = fields1.concat(fields2);
			
			isEmpty = true;
			_.each(fields, function(field) {
				f = view.$(field);
				v = f.val();
				
				// Case checkboxes, only when checked
				if ((f.prop('type') == 'checkbox') && (!f.is(':checked')))
				{
					v = null;
				}
				
				if ((_.isArray(v)) && (v.length > 0))
				{
					v = v[0];
				}
					
				if (!_.isEmpty(v))
				{
					isEmpty = false;
				}
			}, this);
			
			return isEmpty;
		}

	};

	var AutocompleteView = Backbone.View.extend({

		initialize : function() {
			var view = this, box = this.$('.XTSearchBox');
	
			this.cache = {};
			this.lastXhr = null;
	
			if ((!box.hasClass('xtd-disabled-typeahead')) && (box.typeahead))
			{
				box.typeahead({
					source : function(query, process) {
						if (query.length >= 3) {
							return view.getEntries(query, process);
						} else {
							return null;
						}
					},
					matcher : function(item) {
						// In SobiPro, we trust
						return true;
					}
				});
			}
		},
	
		getEntries : function(query, process) {
			var view = this;
	
			if (this.cache.hasOwnProperty(query)) {
				return process(this.cache.query);
			}
	
			this.lastXhr = jQuery.ajax({
				url : SobiProUrl.replace('%task%', 'search.suggest'),
				data : {
					term : query,
					sid : view.$('.sectionid').val(),
					tmpl : 'component',
					format : 'raw'
				},
				success : function(data) {
					view.cache.term = data;
					process(data);
				}
			});
		}

	});

	return {

		UiHelper : UiHelper,
		AutocompleteView : AutocompleteView

	};

});
