/**
 * @package     Extly.Modules
 * @subpackage  mod_xtd_extendedsearchsp - Extended Search for SobiPro
 *
 * @author      Prieco S.A.
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */

/* jslint plusplus: true, browser: true, sloppy: true */
/* global jQuery, Request, Joomla, alert, Backbone */

/*

 TEST CASES:
 http://demo.xtdir.com/modules-for-sobipro/extended-search-form-modules.html
 
 SobiPro - Form search (base cases to compare with the module)
 ---------------------
 
 Case 0	 Bookstore							=> 	Found 1 result(s)! Sun Bookstore
 Case 0	 Vivamus							=> 	Found 1 result(s)! Sun Bookstore
 Case 1	 Blank - City Duckburg				=> 	Found 3 result(s)!
 Case 2	 * - City Duckburg					=> 	Found 3 result(s)!
 Case 3	 * - Category Communications 		=>  Found 225 result(s)!
 Case 4	 Blank - Category Communications	=> 	Found 0 result(s)!

 XTDir Extended Search for SobiPro
 --------------------------------- 
 
 Case 0	 Bookstore (default blank)			=> 	Found 1 result(s)! Sun Bookstore
 Case 0	 Bookstore / 2nd Step				=> 	Found 1 result(s)! Sun Bookstore
 Case 0	 Vivamus (default *)				=> 	Found 661 result(s)!
 Case 0	 Blank (default Vivamus)			=> 	Found 661 result(s)! 
 Case 1	 Blank - City Duckburg				=> 	Found 3 result(s)!	TEST OK
 Case 2	 Reset - City Duckburg				=> 	Found 3 result(s)!	TEST OK
 Case 3	 Manually adding *  - City Duckburg	=> 	Found 3 result(s)!	TEST OK
 Case 4	 Default search *  - Category Communications => Found 225 result(s)! TEST OK
 Case 5	 No Default search (Blank)- Category Communications	=> 	Found 0 result(s)!
 Case 6  Blank								=> Found 2011 result(s)!
 Case 7  Vivamus - City Duckburg / 2nd Step	=> Found 1 result(s)!
 Case 7  Category Communications multiple   => Found 225 result(s)! TEST OK

*/

define('xtdsearchsp', [ 'extlycore', 'xtdhelper' ], 
	function (Core, XtdHelper) {
	"use strict";

	var ExtSearchView = Backbone.View.extend({
		
		events : {
			'click .xtdsearchspButton' : 'onSearchSubmit',
			'click .xtdResetButton' : 'onSearchReset'
		},

		initialize : function() {
			
			// Disable submit button default click
			this.$('.xtdsearchspButton').click(function(e) {
				e.preventDefault();
			});
			
			this.$('.btn-group button').click(function(e) {
				e.preventDefault();
			});

			this.$form = this.$('form');
			
			XtdHelper.UiHelper.initChainedCategories(this, this.$form.attr("id"));
			
			if (this.$form.xtautosave) {
				this.$form.xtautosave();
			}
			
			this.setHiddenCategoryField();
		},
		
		setHiddenCategoryField: function() {
			var category = this.$('.xtdcategory'),
				categories = this.$('.sid_list option:selected'), v;	

			this.$('.xtd_select_category').val('');
			
			// Category - SobiPro Layout
			if ( (category.length !== 0) && (!_.isEmpty(v = category.val())) ) {
				this.$('.xtd_select_category').val(v);					
			}
			
			// Categories
			if (!_.isEmpty(categories)) {
				categories = _.filter(categories, function(f) {
					return (!_.isEmpty(f.value));
				});
				category = _.last(categories);
				
				if (category) {
					v = this.$(category).val().replace('spc', '');
					this.$('.xtd_select_category').val(v);
				}
			}			
		},
		
		setSearchPhrase: function() {
			var v, spsearchphrase, dataref;
			
			spsearchphrase = this.$('.btn-group .active');
			dataref = '.' + spsearchphrase.attr('data-ref');
			v = spsearchphrase.val();
			this.$(dataref).val(v);			
		},
		
		onSearchSubmit: function() {
			var query, fieldsEmpty;
			
			this.setHiddenCategoryField();
			this.setSearchPhrase();
			
			XtdHelper.UiHelper.onSearchSubmit(this, this.$form);
		},
		
		onSearchReset: function() {
			this.$('form')[0].reset();
			this.$('.sid_list1').trigger('change');
			
			return false;
		}
		
	});
	
	var ExtSectionSelectView = Backbone.View.extend({
		
		events : {
			'change .SPSectionSelect' : 'onSectionSelect'
		},
		
		onSectionSelect: function() {
			var sid = this.$('.SPSectionSelect').val();
			this.$('.sectionid').val(sid);
		}		
	});
	
	// Just to be sure
	jQuery(document).ready(function() {
		var forms = jQuery('.xtdsearchsp');	
		_.each(forms, function(form)
				{
					var extSearchView = new ExtSearchView({el : form}),
						autocompleteView = new XtdHelper.AutocompleteView({el : form}),
						extSectionSelectView = new ExtSectionSelectView({el : form});
				});
	});

});
