<?php

/**
 * @package     Extly.Modules
 * @subpackage  mod_xtd_promoentries - XTDir Promoted Entries
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/*
 * Based on --- SobiPro Entries Module ---
 * Author
 * ------
 *   Name: Sigrid Suski & Radek Suski, Sigsiu.NET GmbH
 *   Email: sobi[at]sigsiu.net
 *   Url: http://www.Sigsiu.NET
 *   copyright Copyright (C) 2006 - 2013 Sigsiu.NET GmbH (http://www.sigsiu.net). All rights reserved.
 *   license GNU/GPL Version 3
 *
 * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation, and under the additional terms according section 7 of GPL v3.
 * See http://www.gnu.org/licenses/gpl.html and http://sobipro.sigsiu.net/licenses.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 */

defined('DS') || define('DS', DIRECTORY_SEPARATOR);
define('MOD_XTD_PROMOENTRIES_ROOT', JPATH_ROOT);
define('MOD_XTD_PROMOENTRIES_PATH', MOD_XTD_PROMOENTRIES_ROOT . '/components/com_sobipro');
require_once MOD_XTD_PROMOENTRIES_PATH . '/lib/cms/joomla_common/elements/spsection.php';

/**
 * JElementXtdElements
 *
 * @package     Extly.Modules
 * @subpackage  com_xtdir
 * @since       1.0
 */
class JElementXtdElements extends JElementSPSection
{
	protected $settings = null;

	protected $sections = array();

	/**
	 * setSettings
	 *
	 * @param   object  $setting  Params
	 *
	 * @return  void
	 */
	public function setSettings($setting)
	{
		$this->settings = $setting;
	}

	/**
	 * determineTask
	 *
	 * @return  void
	 */
	protected function determineTask()
	{
		try
		{
			$this->sections = SPFactory::db()->select('*', 'spdb_object', array(
							'oType' => 'section'
			), 'id')->loadObjectList('id');
		}
		catch (SPException $x)
		{
		}
	}

	/**
	 * getInstance
	 *
	 * @return  object
	 */
	public static function getInstance()
	{
		static $instance = null;

		if (!($instance instanceof self))
		{
			$instance = new self;
		}

		return $instance;
	}

	/**
	 * field
	 *
	 * @param   array  &$data  Params
	 *
	 * @return  void
	 */
	protected function field(&$data)
	{
		$f = SPConfig::fields(
				$this->settings()->get('section'),
				array(
						'inbox',
						'select',
						'calendar',
						'promoinformation'
			)
		);

		if (count($f))
		{
			foreach ($f as $field => $label)
			{
				$data[$field] = $label;
			}
		}
	}

	/**
	 * ordering
	 *
	 * @param   string  $selected  Params
	 *
	 * @return  object
	 */
	private function ordering($selected = null)
	{
		$fData = array(
						'counter.asc' => Sobi::Txt('SEC.CFG.ENTRY_ORDER_BY_POPULARITY_ASCENDING'),
						'counter.desc' => Sobi::Txt('SEC.CFG.ENTRY_ORDER_BY_POPULARITY_DESCENDING'),
						'createdTime.asc' => Sobi::Txt('SEC.CFG.ENTRY_ORDER_BY_CREATION_DATE_ASC'),
						'createdTime.desc' => Sobi::Txt('SEC.CFG.ENTRY_ORDER_BY_CREATION_DATE_DESC'),
						'updatedTime.asc' => Sobi::Txt('SEC.CFG.ENTRY_ORDER_BY_UPDATE_DATE_ASC'),
						'updatedTime.desc' => Sobi::Txt('SEC.CFG.ENTRY_ORDER_BY_UPDATE_DATE_DESC'),
						'validUntil.asc' => Sobi::Txt('SEC.CFG.ENTRY_ORDER_BY_EXPIRATION_DATE_ASC'),
						'validUntil.desc' => Sobi::Txt('SEC.CFG.ENTRY_ORDER_BY_EXPIRATION_DATE_DESC'),
						'RAND()' => JText::_('MOD_XTD_PROMOENTRIES_MOD_RANDOM')
		);
		$this->field($fData[JText::_('MOD_XTD_PROMOENTRIES_ORDER_BY_FIELDS')]);

		return SPHtml_Input::select('jform[params][spOrder]', $fData, $selected);
	}

	/**
	 * tplFile
	 *
	 * @param   string  $selected  Params
	 * @param   bool    $modal     Params
	 *
	 * @return  object
	 */
	private function tplFile($selected = null, $modal = false)
	{
		$dir = MOD_XTD_PROMOENTRIES_PATH . '/usr/templates/front/modules/xtdpentries/' . ($modal ? 'modal/' : null);

		if (!(file_exists($dir)) || (count(scandir($dir)) < 3))
		{
			$this->installTpl($modal);
		}

		if (!(file_exists(MOD_XTD_PROMOENTRIES_PATH . '/usr/templates/front/modules/xtdpentries/default2.xsl')))
		{
			$this->installTpl(false);
		}

		$files = scandir(MOD_XTD_PROMOENTRIES_PATH . '/usr/templates/front/modules/xtdpentries/' . ($modal ? 'modal/' : null));
		$tpls = array();

		if (count($files))
		{
			foreach ($files as $file)
			{
				$stack = explode('.', $file);

				if (array_pop($stack) == 'xsl')
				{
					$tpls[$file] = $file;
				}
			}
		}

		if (!($modal))
		{
			return SPHtml_Input::select('jform[params][tplFile]', $tpls, $selected);
		}
		else
		{
			return SPHtml_Input::select('jform[params][modalTemplate]', $tpls, $selected);
		}
	}

	/**
	 * tplFile
	 *
	 * @param   bool  $modal  Params
	 *
	 * @return  void
	 */
	private function installTpl($modal = false)
	{
		$dir = MOD_XTD_PROMOENTRIES_PATH . '/usr/templates/front/modules/xtdpentries/' . ($modal ? 'modal/' : null);
		SPFs::mkDir($dir);
		$files = array(
						'mod_xtd_promoentries.js' => 'components/com_sobipro/lib/js/mod_xtd_promoentries.js',
						'mod_xtd_promoentries-min.js' => 'components/com_sobipro/lib/js/mod_xtd_promoentries-min.js',
						'xtdpmod.php' => 'components/com_sobipro/lib/ctrl/xtdpmod.php',
						'xtdvpmod.php' => 'components/com_sobipro/lib/views/xtdvpmod.php'
		);

		foreach ($files as $from => $to)
		{
			if (SPFs::exists(MOD_XTD_PROMOENTRIES_ROOT . '/' . $to))
			{
				SPFs::delete(MOD_XTD_PROMOENTRIES_ROOT . '/' . $to);
			}

			SPFs::copy(dirname(__FILE__) . '/install/' . $from, MOD_XTD_PROMOENTRIES_ROOT . '/' . $to);
		}

		if (!($modal))
		{
			$files = scandir(dirname(__FILE__) . '/install/tmpl/');

			if (count($files))
			{
				foreach ($files as $file)
				{
					if (strstr($file, '.xsl'))
					{
						SPFs::copy(dirname(__FILE__) . '/install/tmpl/' . $file, MOD_XTD_PROMOENTRIES_PATH . '/usr/templates/front/modules/xtdpentries/' . $file);
					}
				}
			}
		}
		else
		{
			$files = scandir(dirname(__FILE__) . '/install/tmpl/modal/');

			if (count($files))
			{
				foreach ($files as $file)
				{
					if (strstr($file, '.xsl'))
					{
						SPFs::copy(dirname(__FILE__) . '/install/tmpl/modal/' . $file, MOD_XTD_PROMOENTRIES_PATH . '/usr/templates/front/modules/xtdpentries/modal/' . $file);
					}
				}
			}
		}
	}

	/**
	 * settings
	 *
	 * @return  object
	 */
	protected function settings()
	{
		static $settings = null;

		if (!($settings))
		{
			$settings = new JRegistry($this->settings);
		}

		return $settings;
	}

	/**
	 * fetchElement
	 *
	 * @param   string  $name    Params
	 * @param   object  &$label  Params
	 *
	 * @return  object
	 */
	public function fetchElement($name, &$label)
	{
		$sid = $this->settings()->get('sid');
		$this->oType = 'section';

		switch ($name)
		{
			case 'sid':
				$params = array(
								'id' => 'sid',
								'size' => 5,
								'class' => 'text_area',
								'style' => 'text-align: center;',
								'readonly' => 'readonly'
				);

				return SPHtml_Input::text('jform[params][sid]', $sid, $params);
				break;
			case 'tplFile':
			case 'modalTemplate':
				return $this->tplFile($this->settings()->get('tplFile'), $name == 'modalTemplate');
				break;
			case 'spOrder':
				return $this->ordering($this->settings()->get('spOrder'));
				break;
			case 'cid':
				if (!(in_array($sid, array_keys($this->sections))))
				{
					$catName = SPLang::translateObject(
							$sid,
							array(
									'name'
					)
					);

					if (isset($catName[$sid]['value']))
					{
						$this->oName = $catName[$sid]['value'];
						$this->oType = 'category';
					}
				}

				return $this->getCat();
				break;
			default:
				$sections = array();

				if (count($this->sections))
				{
					$sections[] = Sobi::Txt('SELECT_SECTION');

					foreach ($this->sections as $section)
					{
						if (Sobi::Can('section', 'access', 'valid', $section->id))
						{
							$s = SPFactory::Model('section');
							$s->extend($section);
							$sections[$s->get('id')] = $s->get('name');
						}
					}
				}

				$params = array(
								'id' => 'spsection',
								'class' => 'text_area required'
				);

				return SPHtml_Input::select('jform[params][section]', $sections, $this->settings()->get('section'), false, $params);
				break;
		}
	}
}
