<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

// Check for component
if (!JComponentHelper::getComponent('com_sobipro', true)->enabled)
{
	if (JError::$legacy)
	{
		JError::raiseWarning('5', 'SobiPro Component is not installed or not enabled.');
	}
	else
	{
		JFactory::getApplication()->enqueueMessage('SobiPro Component is not installed or not enabled.', 'warning');
	}

	return;
}

include_once JPATH_ADMINISTRATOR . '/components/com_xtdir/api/xtdirapi.php';
include_once JPATH_ADMINISTRATOR . '/components/com_xtdir/api/sobipro_api.php';

/**
 * PlgSearchXtdirsobipro class.
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
*/
class PlgSearchXtdirsobipro extends JPlugin
{
	private $_searcher = null;

	/**
	 * Constructor
	 *
	 * @param   object  &$subject  The object to observe
	 * @param   array   $config    An array that holds the plugin configuration
	 *
	 * @since   1.0
	 *
	 */
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);
		$this->loadLanguage();

		$this->_searcher = new Searcher($this->params);
	}

	/**
	 * onContentSearchAreas event
	 *
	 * @return	array An array of search areas.
	 *
	 * @since	1.0
	 */
	public function onContentSearchAreas()
	{
		$title = $this->params->def('title', JText::_('PLG_SEARCH_XTDIRSOBIPRO_ITEMS'));

		$areas = array();
		$areas['xtdirsobipro'] = $title;

		return $areas;
	}

	/**
	 * onContentSearch event
	 *
	 * @param   string  $text      Text to search
	 * @param   string  $phrase    How to search
	 * @param   string  $ordering  How to order
	 * @param   string  $areas     Where to search
	 *
	 * @return	array Result - The sql must return the following fields that are used in a common display
	 * routine: href, title, section, created, text, browsernav
	 *
	 * @since	1.0
	 */
	public function onContentSearch($text, $phrase = '', $ordering = '', $areas = null)
	{
		$rows = array();

		if (is_array($areas))
		{
			if (!array_intersect($areas, array_keys($this->onContentSearchAreas())))
			{
				return $rows;
			}
		}

		return $this->_searcher->execute($text, $phrase, $ordering);
	}
}
